/*--------------------------------------------------------------------------------------------------
Set up

Initial characteristics for replication the analysis

	Outline:

	1. standardize settings
	2. path
	3. packages
	4. run analysis
--------------------------------------------------------------------------------------------------*/

*---------------------------------------------------------------------------------------------------
**# 1. standardize settings #

* initial setting
set more off , //perm
clear all
version 17.0
set varabbrev on

* set seed for replication
set seed 12345
set sortseed 12345

*---------------------------------------------------------------------------------------------------
**# 2. path #

global projectfolder "C:/Users/wb532468/OneDrive - WBG/Rep-checks-VDI/20231005-centralized-accountability-v2"

* new folders: folders already created in the package

/*
mkdir "$projectfolder/analysis/processed/stacked" // new folder for stacked files

*figures

mkdir "$projectfolder/analysis/figures" // new folder for figures
mkdir "$projectfolder/analysis/figures/descriptive" // new folder for descriptive figures
mkdir "$projectfolder/analysis/figures/causal" // new folder for causal figures
mkdir "$projectfolder/analysis/figures/quality" // new folder for quality analysis figures

*tables

mkdir "$projectfolder/analysis/tables" // new folder for tables
mkdir "$projectfolder/analysis/tables/descriptive" // new folder for descriptive tables
mkdir "$projectfolder/analysis/tables/causal" // new folder for causal tables
mkdir "$projectfolder/analysis/tables/quality" // new folder for quality analysis tables
*/

* globals path for analysis

global analysis "$projectfolder/analysis" // main folder
global data "$analysis/data" // data folder
global code "$analysis/code" // code folder

*---------------------------------------------------------------------------------------------------
**# 3. packages #

sysdir set PLUS "$analysis/adofiles"

/*
if(0){ //change to 1 to RUN. Last update: november 14 / 2023

	* miscelaneous
	
	ssc install egenmore // additional functions for variable calculations
	ssc install outreg2 // storing output for table exporting
	ssc install geodist // calculate gps distances
	ssc install geonear // nearest neighbor calculation
	ssc install group_id // grouping based on shared id
	ssc install jarowinkler // implementation of jarowrinkler: for fuzzy matching
	ssc install palettes // colors in graphs
	net install colrspace, replace from(https://raw.githubusercontent.com/benjann/colrspace/master/) // color management in mata
	ssc install sencode // encode string variable
	
	* network analysis in stata
	
	net from http://nwcommands.org
	net install "nwcommands-ado"
	
	*rd analysis
	
	net install rdrobust, from(https://raw.githubusercontent.com/rdpackages/rdrobust/master/stata) replace
	
	* regression
	
	ssc install bcuse
	*ssc install ftools 
	net install ftools, from("https://raw.githubusercontent.com/sergiocorreia/ftools/master/src/")
	ssc install moremata
	*ssc install reghdfe // high dimensionality fixed effects (all the above required)
	net install reghdfe, from("https://raw.githubusercontent.com/sergiocorreia/reghdfe/master/src/")
	*ssc install ivreghdfe // high dimensionality IV fixed effects
	net install ivreghdfe, from(https://raw.githubusercontent.com/sergiocorreia/ivreghdfe/master/src/)
	ssc install ivreg2 // IV regression
	ssc install ranktest
	
	ssc install did_multiplegt // dif-in-dif for heterogeneous effects (deChaisemartin)
	ssc install eventstudyweights // implement dif-in-dif with staggered adoption (Sun & Abraham)
	net install github, from("https://haghish.github.io/github/") // suplement for eventstudyweights
	github install lsun20/eventstudyinteract // suplement for eventstudyweights
	ssc install synth // implement synthetic control (build synthetic control group)

	* figures

	ssc install addplot // add plot to coefplot
	net install panelview, all replace from("https://yiqingxu.org/packages/panelview_stata") // panelview
	
	/* similar problem to panelview */
	ssc install grstyle // graph style

	ssc install tabplot // summary graphs by categories
	ssc install beamplot // dotplots and binscatter
	ssc install coefplot // plot of coefficients
	net install grc1leg, from(http://www.stata.com/users/vwiggins/) // management of legends in graph
	
	* tables 
	ssc install estout // format of tables
	ssc install labutil // table labels management
	
	* shrinking method
	ssc install ranktest // test rank of matrix
	ssc install fese // calculate standard errors for fixed effects
	
	*fese_fast and ebayes downloaded from https://sacarny.com/programs/ and manually set in adofiles folder

}
*/

* always run this: some packages have problems reading mata
capture mata: mata drop m_calckw()
capture mata: mata drop m_omega()
capture mata: mata drop ms_vcvorthog()
capture mata: mata drop s_vkernel()
mata: mata mlib index

*---------------------------------------------------------------------------------------------------
**# 4. run analysis #

* do files

// data cleaning
do "$code/1_stack_data.do"

// descriptive analysis
do "$code/2_descriptive.do"

// main analysis
do "$code/3_main_regressions.do"
