/*--------------------------------------------------------------------------------------------------
Descriptive analysis

	Outline:
	
	1. descriptive statistics  school and mar level
		- Table 1 - Panel A, B
		
	2. measures of variation
		- Table 2
		
	3. descriptive statistics district level
		- Table 1 - Panel C
		- Figure 3
		
	4. evolution of outcomes in relative time
		- Figure 4
		
	5. data validation: distribution and errors in reporting
		- Figure A2
		
	6. data validation: evolution of outcomes by district
		- Figure B1

--------------------------------------------------------------------------------------------------*/

*---------------------------------------------------------------------------------------------------
**# 1. descriptive statistics school and mar level #

* Table 1 Panel A

if(1){

* we use the balanced dataset
use "$analysis/processed/cc_datapacks.dta", clear
drop if wing == 3

//general

label var totalteachers "Number of teachers"
label var students "Number of students"
estpost tabstat totalteachers students, statistics(mean p50 sd count) columns(statistics)
qui esttab . using "$analysis/tables/descriptive/1_sumstatsgeneral_school.tex", ///
		cells("mean(fmt(%11.2gc) label(Mean)) p50(fmt(%11.2gc) label(Median)) sd(fmt(%11.2gc) label(Std. Dev.)) count(fmt(%11.2gc) label(Obs))") ///
		nomtitle nonumber label replace noobs  booktabs compress

//outcomes

label var tpres "Teacher presence"
label var satt "Student attendance"
label var fac "Functional facilities"
label var math "Math score"
label var eng "Englsih score"
label var urdu "Urdu score"
label define f 1 "Flag" 0 "No flag"
local vars tpres satt fac math eng urdu
foreach v of local vars{	
	
	label values treat_`v'_ f
	estpost tabstat `v' , by(treat_`v'_) statistics(mean p50 sd count) columns(statistics) nototal 
	esttab . using "$analysis/tables/descriptive/1_sumstats`v'_school.tex", ///
		cells("mean(fmt(%11.2gc) label(Mean)) p50(fmt(%11.2gc) label(Median)) sd(fmt(%11.2gc) label(Std. Dev.)) count(fmt(%11.2gc) label(Obs))") ///
		uns nonumber label replace noobs booktabs compress	
		
} // end of loop

}

*Table 1 Panel B

if(1){
	
// data arrangement for table

* we use the balanced dataset
use "$analysis/processed/cc_datapacks.dta", clear

*create variable for number of schools and proportion not in wing 3
sort mar date_treat emis
gen random = runiform()
sort mar date_treat emis random
bys mar date_treat emis (random): gen ord = _n
keep if ord == 1
drop random ord

gen temp = 1
bys mar date_treat: egen numschools = sum(temp)
drop temp
gen temp = 1 if wing != 3
bys mar date_treat: egen propnowingchools = sum(temp)
drop temp
gen propschools = (propnowingchools/numschools)*100

preserve
*drop if wing == 3
collapse (max) propschools numschools, ///
		by(mar date_treat treat_tpres_ treat_satt_ treat_fac_ treat_math_  ///
		   treat_eng_ treat_urdu_)
	
//general

label var numschools "Number of schools"
label var propschools "Proportion elementary"
estpost tabstat numschools propschools, statistics(mean p50 sd count) columns(statistics)
qui esttab . using "$analysis/tables/descriptive/1_sumstatsgeneral_mar.tex", ///
		cells("mean(fmt(%11.2gc) label(Mean)) p50(fmt(%11.2gc) label(Median)) sd(fmt(%11.2gc) label(Std. Dev.)) count(fmt(%11.2gc) label(Obs))") ///
		nomtitle nonumber label replace noobs  booktabs compress
restore

preserve

drop if wing == 3
collapse (mean) mar_tpres mar_satt mar_fac ///
			                                 mar_math mar_eng mar_urdu, ///
		by(mar date_treat treat_tpres_ treat_satt_ treat_fac_ treat_math_  ///
		   treat_eng_ treat_urdu_)
	
//outcomes
label var mar_tpres "Teacher presence"
label var mar_satt "Student attendance"
label var mar_fac "Functional facilities"
label var mar_math "Math score"
label var mar_eng "Englsih score"
label var mar_urdu "Urdu score
local vars tpres satt fac math eng urdu

foreach v of local vars{	
	estpost tabstat mar_`v', by(treat_`v'_) statistics(mean p50 sd count) columns(statistics) 
	esttab . using "$analysis/tables/descriptive/1_sumstats`v'_mar.tex", ///
		cells("mean(fmt(%11.2gc) label(Mean)) p50(fmt(%11.2gc) label(Median)) sd(fmt(%11.2gc) label(Std. Dev.)) count(fmt(%11.2gc) label(Obs))") ///
		nomtitle nonumber label replace noobs  booktabs compress

} // end of loop

restore

}

*---------------------------------------------------------------------------------------------------
**# 2. measures of variation #

* Table 2

if(1){
	
use "$analysis/processed/cc_datapacks.dta", clear
keep if wing != 3

*collapse all schools time as an average of each variable: then at the school level
collapse (mean) tpres satt fac math eng urdu, by(emis)

*create for each category the quintiles
local vars tpres satt fac math eng urdu
foreach v of local vars{
	xtile quint_`v' = `v', n(5)	
	
} // end loop

*estimate variation table for each variable
/*as before we create the paper table based on this tables that require further formating*/
local vars tpres satt fac math eng urdu
foreach v of local vars{	
	estpost tabstat `v', by(quint_`v') statistics(sd mean count) columns(statistics) 
	esttab . using "$analysis/tables/descriptive/2_sumvariation`v'_school.tex", ///
		cells("sd(fmt(%11.2gc) label(Std. Dev.)) mean(fmt(%11.2gc) label(Mean)) count(fmt(%11.2gc) label(Obs))") ///
		nomtitle nonumber label replace noobs  booktabs compress
		
} // end loop


}

*---------------------------------------------------------------------------------------------------
**# 3. descriptive statistics district level #

* Table 1 Panel C

use "$data/ranking", replace

if(1){
	
// sumstats of overall score

preserve
 
*ranking variable
gen D = (rank_position < 6) 
replace D = 2 if rank_position > 31

* drop outside the extremes
drop if D == 0
estpost tabstat overallscore, by(D) statistics(mean p50 sd count) columns(statistics)
qui esttab . using "$analysis/tables/descriptive/3_sumstatsscore_district.tex", ///
			cells("mean(fmt(%11.2gc) label(Mean)) p50(fmt(%11.2gc) label(Median)) sd(fmt(%11.2gc) label(Std. Dev.)) count(fmt(%11.2gc) label(Obs))") ///
			nomtitle nonumber label replace noobs  booktabs compress
restore 

// sumstats of change of positions

preserve 
	*ranking variable
gen D = (rank_position < 6) 
replace D = 2 if rank_position > 31
bys district (date_treat): gen old_D = D[_n-1]
gen new_top = (old_D != D & D == 1)
gen new_bottom = (old_D != D & D == 2)
	* drop outside the extremes
estpost tabstat new_top new_bottom, statistics(mean p50 sd count) columns(statistics)
qui esttab . using "$analysis/tables/descriptive/3_sumstatsrank_district.tex", ///
			cells("mean(fmt(%11.2gc) label(Mean)) p50(fmt(%11.2gc) label(Median)) sd(fmt(%11.2gc) label(Std. Dev.)) count(fmt(%11.2gc) label(Obs))") ///
			nomtitle nonumber label replace noobs  booktabs compress
restore 

}

* Figure 3

if(1){
	
preserve 

	*estimate quartiles
gen cuart = .
levelsof date_treat
	foreach d in `r(levels)'{
		xtile x = overallscore if date_treat == `d', nq(5)
		replace cuart = x if date_treat == `d'
		drop x
	}
	
sort district cuart
gen x = cuart
bys district: egen total = sum(overall)
bys date_treat (total): gen id = _n

	*panelview
colorpalette "red" "orange" "yellow" "lime" "green", nograph
panelview x cuart if date_treat < tm(2015m3), i(id) t(date_treat) type(treat) ///
	ytitle("District performance position") xtitle("Quarterly meetings") graphregion(color(white)) ///
	ylabel(1 "Worst performer" 36 "Best performer",  labsize(small)) ///
	xlabel(1 "2011m12" 2 "2012m3" 3 "2012m5" 4 "2012m9" 5 "2012m12" 6 "2013m3" 7 "2013m6" ///
		8 "2013m9" 9 "2013m12" 10 "2014m3" 11 "2014m6" 12 "2014m9" 13 "2014m12", labsize(small)) ///
	mycolor(`r(p)') ///
	legend(order(1 "1st quintile" 2 "2nd quintile" 3 "3rd quintile" 4 "4th quintile") size(small)) 
	
graph export "$analysis/figures/descriptive/3_scoredist_district.pdf", replace

restore

}

*---------------------------------------------------------------------------------------------------
**# 4. evolution of outcomes in relative time #

* Figure 4 

global treatments tpres satt fac math eng urdu // Panel A, B, C, D, E, F
foreach v of global treatments{
	
/*we use stacked data with 5 post periods after the punishment
note that we have to iterate over each variable as we created different datasets for each one
*/

use "$analysis/processed/stacked/cc_stackdata5_`v'.dta", clear

/*functional facilities evolution is "weird", and each variable has its characteristics, 
we plot them separately*/

	* evolution full data
	preserve
	collapse (mean) mar_`v' [aw = avg_totalreportedteachers_mar], ///
		by(flagged relativetime)
	gen full = 1
	tempfile full
	save `full'
	restore 
		
	* evolution rd data
	preserve
	keep if rd1 == 1
	collapse (mean) mar_`v' [aw = avg_totalreportedteachers_mar], ///
		by(flagged relativetime)
			
	gen full = 0
	tempfile rd
	save `rd'
	restore 

	
	* merge both to graph
	preserve
	use `full', clear
	append using `rd'
	
	//define parameters
	
	if("`v'" == "fac"){
		
		local ymin = 80
		local bet = 2		
		local height = 83
		
	}
	else if("`v'" == "tpres" | "`v'" == "satt"){
		
		local ymin = 80
		local bet = 2
		local height = 96
		
	}
	else{
		
		local ymin = 60
		local bet = 5		
		local height = 93
		
	}


		
	* graph everything
	twoway (line mar_`v' relativetime if flagged == 1 & full == 1, ///
			lcol("navy") lwidth(0.5)) ///
		(line mar_`v' relativetime if flagged == 0 & full == 1, ///
			lcol("navy") lpattern(dash) lwidth(0.5)) ///
		(line mar_`v' relativetime if flagged == 1 & full == 0, ///
			lcol("cranberry") lwidth(0.5)) ///
		(line mar_`v' relativetime if flagged == 0 & full == 0, ///
			lcol("cranberry") lpattern(dash) lwidth(0.5)) ///
		(scatteri 100 -0.5 100 0.49, recast(area) lwidth(none) col(gs14%50)) ///
		(scatteri 100 0.5 100 1.49, recast(area) lwidth(none) col(gs12%50)) ///
		(scatteri 100 1.5 100 6.49, recast(area) lwidth(none) col(gs8%50)), ///
		text(`height' 0 "{bf: Flag} " `height' 1 "{bf:Punish}" `height' 4 "{bf:After}" "{bf:flag}", size(large) ///
			orient(vertical) placement(c)) ///
		graphregion(color(white)) xtitle("Period", size(large)) ytitle("Average outcome (%)", size(large)) ///
		xlabel(-4(1)6) yscale(r(`ymin' 100)) ylabel(`ymin'(`bet')100) ///
		legend(order(1 "Flag (Full)" 2 "No flag (Full)" 3 "Flag (Threshold)" 4 "No flag (Threshold)") size(large) region(lstyle(none)))
			
	graph export "$analysis/figures/descriptive/1_`v'evol.pdf", replace
	restore
	
	}	

*---------------------------------------------------------------------------------------------------
**# 5. data validation: distribution and errors in reporting #

* Figure A2
 				
/*under the assumption that the monthly data is not as independent as the census, we compare both 
data sources to ensure the quality of the data. For this we only have number of teachers posted, 
enrollment and functional facilities
 
We proceed in the following way:
 
For each year between 2012 and 2017 we compare the distribution
For each year we plot the differences
 
We compare the annual census with each november as is the closest date to the date of census 
collection

notes on this: census data for all schools, but monthly for near of the 80% per markaz randonly,
so we only check those in both
*/
			
*census data: independently collected once a year

if(1){
	
preserve

use "$data/anualschoolcensus", clear

*standardize columns
rename emiscode emis
rename year_new year
rename total_teachers_new totalteachers
rename total_school_enrollment_new totalenrol
rename electricity_new elec_f
rename drinking_water_new dw_f
rename toilet_new toilet_f
rename building_new bw_f
keep emis year totalteachers totalenrol elec_f dw_f toilet_f bw_f 
sort emis year
gen random = runiform()
sort emis year random
bys emis year (random): gen ord = _n
keep if ord == 1
drop random ord
drop if year < 2012
gen census = 1
tempfile temp
save `temp'

restore

*original monthly data: not cleaned or adjusted version
use "$data/cc_mea.dta", clear 

*create necessary columns
egen totalenrol = rowtotal(cls1enrol cls2enrol cls3enrol cls4enrol cls5enrol cls6enrol cls7enrol ///
	cls8enrol cls9enrol cls10enrol cls11enrol cls12enrol)
replace totalenrol = . if totalenrol == 0
keep if month == 11 
keep emis year totalteachers totalenrol elec_f dw_f toilet_f bw_f 

sort emis year
gen random = runiform()
sort emis year random
bys emis year (random): gen ord = _n
keep if ord == 1
drop random ord
drop if year < 2012 | year > 2017
gen census = 0

*append both data sources
append using `temp'

*keep only those appearing in both samples
bys emis year: egen c = count(census)
keep if c == 2
drop c

local vars tpres satt fac
foreach v of local vars{
	
	if("`v'" == "tpres"){
			
		* teacher presence

			*distributions of errors in the reporting
		preserve

		bys emis year (census): gen diff = totalteachers[_n] - totalteachers[_n+1]
		replace diff = (diff/totalteachers)*100
		drop if diff == .
		sum diff, d
		drop if diff < `r(p1)' | diff > `r(p99)'
		twoway (hist diff, color(blue%40)), by(year, graphregion(fcolor(white))) ///
			graphregion(color(white)) plotregion(color(white)) ///
			xtitle("% difference in number of teacher (PMIU-Census)", size(large)) ///
			ytitle("Density", size(large))
		graph export "$analysis/figures/descriptive/4_diffdata_tpres.pdf", replace
		restore 

			*distribution of overlapping variables
		preserve
		replace totalteachers = ln(totalteachers + 1)
		*plot distribution of both variables
		twoway (kdensity totalteachers if census == 1, color(blue%40) recast(area)) ///
		(kdensity totalteachers if census == 0, color(red%20) recast(area)), ///
			by(year , graphregion(fcolor(white))) ///
			graphregion(color(white)) plotregion(color(white)) ///
			xtitle("Number of teachers (logs + 1)", size(large)) ytitle("Density", size(large)) legend(order(1 "Census" 2 "PMIU") size(large))	
		graph export "$analysis/figures/descriptive/4_distdata_tpres.pdf", replace
		restore			

	}
	else if("`v'" == "satt"){
			
		* student attendance
				*distributions of errors in the reporting
		preserve
		bys emis year (census): gen diff = totalenrol[_n] - totalenrol[_n+1]
		replace diff = (diff/totalenrol)*100
		drop if diff == .
		sum diff, d
		drop if diff < `r(p1)' | diff > `r(p99)'
		twoway (hist diff, color(blue%40)), by(year, graphregion(fcolor(white))) ///
			graphregion(fcolor(white)) xtitle("% difference in enrolled students (PMIU-Census)", size(large)) ///
			ytitle("Density", size(large)) 
		graph export "$analysis/figures/descriptive/4_diffdata_satt.pdf", replace
		restore

			*distribution of overlapping variables
		preserve
		replace totalenrol = ln(totalenrol + 1)
		*plot distribution of both variables
		twoway (kdensity totalenrol if census == 1, color(blue%40) recast(area)) ///
			(kdensity totalenrol if census == 0, color(red%30) recast(area)), ///
			by(year , graphregion(fcolor(white))) ///
			graphregion(color(white)) xtitle("Number of students (logs + 1)", size(large)) ///
			ytitle("Density", size(large)) legend(order(1 "Census" 2 "PMIU") size(large))						
		graph export "$analysis/figures/descriptive/4_distdata_satt.pdf", replace
		restore
			
	}
	else if("`v'" == "fac"){
			
		/*functional facilities is acatyegorical variable, then we only plot the percentage of 
		coincidence*/
			
		preserve
		local vars elec_f dw_f bw_f toilet_f
		foreach v of local vars{
			bys emis year (census): gen coincide_`v' = (`v'[_n] == `v'[_n+1])				
		}

		local vars elec_f dw_f bw_f toilet_f
		foreach v of local vars{	
			bys emis year (census): egen coincide_`v'_max = max(coincide_`v')						
		}

		drop if census == 1

		gen counter = 1

		collapse (sum) coincide_elec_f_max coincide_dw_f_max coincide_bw_f_max ///
			coincide_toilet_f_max counter, by(year)
		local vars elec_f dw_f bw_f toilet_f
		foreach v of local vars{	
			gen perc_`v' = (coincide_`v'_max/counter)*100 						
		}

		graph bar perc_elec_f perc_dw_f perc_toilet_f perc_bw_f, by(year , graphregion(fcolor(white))) ///
			ytitle("% coincidence - Census and PMIU", size(large)) ///
			legend(order(1 "Electricity" 2 "Drink water" 3 "Toilet" 4 "Building" ) size(large))
		graph export "$analysis/figures/descriptive/4_diffdata_fac.pdf", replace
		restore

	}
	
}

}

*---------------------------------------------------------------------------------------------------
**# 6. data validation: evolution of outcomes by district #

* Figure B1

/*we use data of external sources (ASER Pakistan) to explore the evolution of 
outcomes*/

if(1){
	
use "$data/asertime", clear
	
* figures

twoway (connect tpres year if province == "Punjab", lwidth(thick)) ///
	(connect tpres year if province == "Sindh", lcol(navy%70) mcol(navy%70) lwidth(thin) msize(small) msymbol(triangle)) ///
	(connect tpres year if province == "Balochistan", lcol(blue%70) mcol(blue%70) lwidth(thin) msize(small) msymbol(diamond)) ///
	(connect tpres year if province == "Gilgit-Baltistan", lcol(midblue%70) mcol(midblue%70) lwidth(thin) msize(small) msymbol(square)) ///
	(connect tpres year if province == "Islamabad - ICT", lcol(lavender%70) mcol(lavender%70) msize(small) lpattern(dash)) ///
	(connect tpres year if province == "Khyber Pakhtunkhwa", lcol(ebblue%70) mcol(ebblue%70) msize(small) lpattern(dash) msymbol(triangle)) ///
	(connect tpres year if province == "FATA", lcol(emidblue%70) mcol(emidblue%70) msize(small) lpattern(dash) msymbol(diamond)) ///
	(connect tpres year if province == "AJK", lcol(gs11%70) mcol(gs11%70) msize(small) lpattern(dash) msymbol(square)), ///	
	ytitle("Average teacher presence", size(large)) yscale(r(60 100) titlegap(*10)) ylabel(60(5)100)  ///
	xlabel(2010(1)2016) xtitle(Year, size(large)) xline(2011.5) ///
	legend(order(1 "Punjab" 2 "Sindh" 3 "Balochistan" 4 "GB" ///
		5 "Islamabad" 6 "KP" 7 "FATA" 8 "AJK") col(4) size(large) position(6)) ///
	graphregion(col(white))
	
graph export "$analysis/figures/descriptive/11_trend_tpres.pdf", replace

twoway (connect satt year if province == "Punjab", lwidth(thick)) ///
	(connect satt year if province == "Sindh", lcol(navy%70) mcol(navy%70) lwidth(thin) msize(small) msymbol(triangle)) ///
	(connect satt year if province == "Balochistan", lcol(blue%70) mcol(blue%70) lwidth(thin) msize(small) msymbol(diamond)) ///
	(connect satt year if province == "Gilgit-Baltistan", lcol(midblue%70) mcol(midblue%70) lwidth(thin) msize(small) msymbol(square)) ///
	(connect satt year if province == "Islamabad - ICT" & year < 2016, lcol(lavender%70) mcol(lavender%70) msize(small) lpattern(dash)) ///
	(connect satt year if province == "Khyber Pakhtunkhwa", lcol(ebblue%70) mcol(ebblue%70) msize(small) lpattern(dash) msymbol(triangle)) ///
	(connect satt year if province == "FATA", lcol(emidblue%70) mcol(emidblue%70) msize(small) lpattern(dash) msymbol(diamond)) ///
	(connect satt year if province == "AJK", lcol(gs11%70) mcol(gs11%70) msize(small) lpattern(dash) msymbol(square)), ///	
	yscale(r(60 100) titlegap(*10)) ylabel(60(5)100) ytitle("Average student attendance", size(large)) ///
	xlabel(2010(1)2016) xtitle(Year, size(large)) xline(2011.5) ///
	legend(order(1 "Punjab" 2 "Sindh" 3 "Balochistan" 4 "GB" ///
		5 "Islamabad" 6 "KP" 7 "FATA" 8 "AJK") col(4) size(large) position(6)) ///
	graphregion(col(white))
	
graph export "$analysis/figures/descriptive/11_trend_satt.pdf", replace

twoway (connect fac year if province == "Punjab", lwidth(thick)) ///
	(connect fac year if province == "Sindh", lcol(navy%70) mcol(navy%70) lwidth(thin) msize(small) msymbol(triangle)) ///
	(connect fac year if province == "Balochistan", lcol(blue%70) mcol(blue%70) lwidth(thin) msize(small) msymbol(diamond)) ///
	(connect fac year if province == "Gilgit-Baltistan", lcol(midblue%70) mcol(midblue%70) lwidth(thin) msize(small) msymbol(square)) ///
	(connect fac year if province == "Islamabad - ICT" & year < 2016, lcol(lavender%70) mcol(lavender%70) msize(small) lpattern(dash)) ///
	(connect fac year if province == "Khyber Pakhtunkhwa", lcol(ebblue%70) mcol(ebblue%70) msize(small) lpattern(dash) msymbol(triangle)) ///
	(connect fac year if province == "FATA", lcol(emidblue%70) mcol(emidblue%70) msize(small) lpattern(dash) msymbol(diamond)) ///
	(connect fac year if province == "AJK", lcol(gs11%70) mcol(gs11%70) msize(small) lpattern(dash) msymbol(square)), ///	
	yscale(r(20 100) titlegap(*10)) ylabel(20(10)100) ytitle("Average functional facilities", size(large)) ///
	xlabel(2010(1)2016) xtitle(Year, size(large)) xline(2011.5) ///
	legend(order(1 "Punjab" 2 "Sindh" 3 "Balochistan" 4 "GB" ///
		5 "Islamabad" 6 "KP" 7 "FATA" 8 "AJK") col(4) size(large) position(6)) ///
	graphregion(col(white))
	
graph export "$analysis/figures/descriptive/11_trend_fac.pdf", replace
	
}















