/*--------------------------------------------------------------------------------------------------
Main analysis

	Outline:

	1. markaz red flagging
		- Figure 5
		- Table 3
		- Figure 7
		- Table B2
		
	2. markaz red flagging - short stack
		- Figure B5
		
	3. markaz red flagging with flagging history
		- Table B1
		
	4. markaz orange flagging stacking
		- Figure B8
		
	5. tehsil red flagging stacking	
		- Figure B9
		
	6. markaz red flagging additional post-periods
		- Figure B4
		
	7. markaz red flagging seasonality
		- Figure B10
		
	8. flagging anticipation
		- Figure B11
		
    9. first time flagged and spillovers to neighbors
		- Figure B2
		- Figure B3
		
   10. alternative difference in differences estimators 
		- Figure B6
		- Figure B7
		
   11. bayesion approach: cumulative effect
		- Figure B12
		
   12. effect on effort
		- Table B5
		- Table B8
		
   13. budget
		- Table B7
		
   14. political alignment
		- Table B4 
		
   15. district ranking
		- Figure 6 
		- Table 4
		- Table B3
		
   16. labor market
		- Table B9
		- Table B10
		
   17. gaming
		- Table B6
   
--------------------------------------------------------------------------------------------------*/

* programs: event study graphs

/* we will be doing a lot of event study graphs, to avoid repeating long chunks of code we set up 
some programs 
*/

/* program for long event studies
- main analysis
- short stack analysis
- flagging history fixed effects
- orange flagging
- tehsil flagging
- first/flag and spillovers to neighbors
- reversion to the mean event studies
- anticipation
*/
capture program drop eventgraph
program define eventgraph

	* arguments of the function
	args yvar full rd xl yl ymin ymax bet height flag punish after typeg
	
	/*
	yvar: outcome
	baseper: base period
	xl: base period line
	yl: zero line
	ymin: min ylabel 
	ymax: max ylabel
	bet: spacing between labels
	height: height of text for periods
	flag punish after: location in x axis of text for each period
	typeg: 
	
		1: long stack
		2: short stack
		3: neighbor effects in spillover specification
		4: reversion to the mean
	*/

	* colorpalette
	
	grstyle init
	grstyle set color "navy" "cranberry" 
	
	* plot
	if(`typeg' == 1){
	
		coefplot (`full', offset(0.1)) /// full sample coefficients
				 (`rd', offset(-0.1)), /// threshold sample coefficient
				 recast(connect) lw(vvthin) /// connected graph
				 baselevels /// include baselevel
				 omitted /// include ommited period (-1)
				 level(95) /// 95% confidence intervals
				 ciopts(recast(rcap)) /// type of confidence intervals
				 vertical /// vertical oriented graph 
				 mcol(%30) /// transparency of points
				 keep( /// interactions to keep in the graph
					  1.flagged#1.time_id 1.flagged#2.time_id 1.flagged#3.time_id ///
					  1.flagged#4.time_id 1.flagged#5.time_id 1.flagged#6.time_id ///
					  1.flagged#7.time_id 1.flagged#8.time_id 1.flagged#9.time_id ///
					  1.flagged#10.time_id 1.flagged#11.time_id ///
				) /// 
				addplot( /// additional graph details to include
					  scatteri `ymax' 4.5 `ymax' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri -`ymin' 4.5 -`ymin' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri `ymax' 5.5 `ymax' 6.49, recast(area) lwidth(none) col(gs12%50) below || ///
					  scatteri -`ymin' 5.5 -`ymin' 6.49, recast(area) lwidth(none) col(gs12%50) below || ///
					  scatteri `ymax' 6.5 `ymax' 11.49, recast(area) lwidth(none) col(gs8%50) below || ///
					  scatteri -`ymin' 6.5 -`ymin' 11.49, recast(area) lwidth(none) col(gs8%50) below ///
				) ///
				yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
				xline(`xl', lpattern("dash") lcol("black")) /// x line for initial period
				yscale(r(-`ymin' `ymax')) /// y scale
				ylabel(-`ymin'(`bet')`ymax') /// y label
				graphregion(color(white)) /// white background
				xtitle("Relative time", size(large)) /// xtitle
				ytitle("Coefficient (pp)", size(large)) /// ytitle
				title("", size(vsmall) col(black)) /// no title
				text( /// text for each post period
					`height' `flag' "{bf: Flag}" `height' `punish' "{bf:Punish}" `height' `after' "{bf:After}" "{bf:flag}", size(large) ///
						orient(vertical) placement(c) ///
				) ///
				coeflabel( /// label of coefficients
					1.flagged#1.time_id = "-4" 1.flagged#2.time_id = "-3" 1.flagged#3.time_id = "-2" ///
					1.flagged#4.time_id = "-1" 1.flagged#5.time_id = "0" 1.flagged#6.time_id = "1" ///
					1.flagged#7.time_id = "2" 1.flagged#8.time_id = "3" 1.flagged#9.time_id = "4"  ///
					1.flagged#10.time_id = "5" 1.flagged#11.time_id = "6" ///
				) ///
				legend(label(8 "Full sample") label(10 "Threshold sample") size(large) region(lstyle(none))) // legend
				
	}
	else if(`typeg' == 2){
		
		coefplot (`full', offset(0.1)) /// full sample coefficients
				 (`rd', offset(-0.1)), /// threshold sample coefficient
				 recast(connect) lw(vvthin) /// connected graph
				 baselevels /// include baselevel
				 omitted /// include ommited period (-1)
				 level(95) /// 95% confidence intervals
				 ciopts(recast(rcap)) /// type of confidence intervals
				 vertical /// vertical oriented graph 
				 mcol(%30) /// transparency of points
				 keep( /// interactions to keep in the graph
					  1.flagged#1.time_id 1.flagged#2.time_id 1.flagged#3.time_id ///
					  1.flagged#4.time_id 1.flagged#5.time_id 1.flagged#6.time_id ///
					  1.flagged#7.time_id 1.flagged#8.time_id ///
				) /// 
				addplot( /// additional graph details to include
					  scatteri `ymax' 4.5 `ymax' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri -`ymin' 4.5 -`ymin' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri `ymax' 5.5 `ymax' 6.49, recast(area) lwidth(none) col(gs12%50) below || ///
					  scatteri -`ymin' 5.5 -`ymin' 6.49, recast(area) lwidth(none) col(gs12%50) below || ///
					  scatteri `ymax' 6.5 `ymax' 8.49, recast(area) lwidth(none) col(gs8%50) below || ///
					  scatteri -`ymin' 6.5 -`ymin' 8.49, recast(area) lwidth(none) col(gs8%50) below ///
				) ///
				yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
				xline(`xl', lpattern("dash") lcol("black")) /// x line for initial period
				yscale(r(-`ymin' `ymax')) /// y scale
				ylabel(-`ymin'(`bet')`ymax') /// y label
				graphregion(color(white)) /// white background
				xtitle("Relative time", size(large)) /// xtitle
				ytitle("Coefficient (pp)", size(large)) /// ytitle
				title("", size(vsmall) col(black)) /// no title
				text( /// text for each post period
					`height' `flag' "{bf: Flag}" `height' `punish' "{bf:Punish}" `height' `after' "{bf:After}" "{bf:flag}", size(large) ///
						orient(vertical) placement(c) ///
				) ///
				coeflabel( /// label of coefficients
					1.flagged#1.time_id = "-4" 1.flagged#2.time_id = "-3" 1.flagged#3.time_id = "-2" ///
					1.flagged#4.time_id = "-1" 1.flagged#5.time_id = "0" 1.flagged#6.time_id = "1" ///
					1.flagged#7.time_id = "2" 1.flagged#8.time_id = "3" ///
				) ///
				legend(label(8 "Full sample") label(10 "Threshold sample") size(large) region(lstyle(none))) // legend
	}
	else if(`typeg' == 3){
		
		coefplot (`full', offset(0.1)) /// full sample coefficients
				 (`rd', offset(-0.1)), /// threshold sample coefficient
				 recast(connect) lw(vvthin) /// connected graph
				 baselevels /// include baselevel
				 omitted /// include ommited period (-1)
				 level(95) /// 95% confidence intervals
				 ciopts(recast(rcap)) /// type of confidence intervals
				 vertical /// vertical oriented graph 
				 mcol(%30) /// transparency of points
				 keep( /// interactions to keep in the graph
					  1.neighmar#1.time_id 1.neighmar#2.time_id 1.neighmar#3.time_id ///
					  1.neighmar#4.time_id 1.neighmar#5.time_id 1.neighmar#6.time_id ///
					  1.neighmar#7.time_id 1.neighmar#8.time_id 1.neighmar#9.time_id ///
					  1.neighmar#10.time_id 1.neighmar#11.time_id ///
				) /// 
				addplot( /// additional graph details to include
					  scatteri `ymax' 4.5 `ymax' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri -`ymin' 4.5 -`ymin' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri `ymax' 5.5 `ymax' 6.49, recast(area) lwidth(none) col(gs12%50) below || ///
					  scatteri -`ymin' 5.5 -`ymin' 6.49, recast(area) lwidth(none) col(gs12%50) below || ///
					  scatteri `ymax' 6.5 `ymax' 11.49, recast(area) lwidth(none) col(gs8%50) below || ///
					  scatteri -`ymin' 6.5 -`ymin' 11.49, recast(area) lwidth(none) col(gs8%50) below ///
				) ///
				yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
				xline(`xl', lpattern("dash") lcol("black")) /// x line for initial period
				yscale(r(-`ymin' `ymax')) /// y scale
				ylabel(-`ymin'(`bet')`ymax') /// y label
				graphregion(color(white)) /// white background
				xtitle("Relative time", size(large)) /// xtitle
				ytitle("Coefficient (pp)", size(large)) /// ytitle
				title("", size(vsmall) col(black)) /// no title
				text( /// text for each post period
					`height' `flag' "{bf: Flag}" `height' `punish' "{bf:Punish}" `height' `after' "{bf:After}" "{bf:flag}", size(large) ///
						orient(vertical) placement(c) ///
				) ///
				coeflabel( /// label of coefficients
					1.neighmar#1.time_id = "-4" 1.neighmar#2.time_id = "-3" 1.neighmar#3.time_id = "-2" ///
					1.neighmar#4.time_id = "-1" 1.neighmar#5.time_id =  "0" 1.neighmar#6.time_id = "1" ///
					1.neighmar#7.time_id =  "2" 1.neighmar#8.time_id =  "3" 1.neighmar#9.time_id = "4"  ///
					1.neighmar#10.time_id = "5" 1.neighmar#11.time_id = "6" ///
				) ///
				legend(label(8 "Full sample") label(10 "Threshold sample") size(large) region(lstyle(none))) // legend
		
	}
	else if(`typeg' == 4){
		
		coefplot (`full', offset(0.1)) /// full sample coefficients
				 (`rd', offset(-0.1)), /// threshold sample coefficient
				 recast(connect) lw(vvthin) /// connected graph
				 baselevels /// include baselevel
				 omitted /// include ommited period (-1)
				 level(95) /// 95% confidence intervals
				 ciopts(recast(rcap)) /// type of confidence intervals
				 vertical /// vertical oriented graph 
				 mcol(%30) /// transparency of points
				 keep( /// interactions to keep in the graph
					  1.flagged#1.time_id 1.flagged#2.time_id 1.flagged#3.time_id ///
					  1.flagged#4.time_id 1.flagged#5.time_id 1.flagged#6.time_id ///
					  1.flagged#7.time_id ///
				) /// 
				addplot( /// additional graph details to include
					  scatteri `ymax' 0.5 `ymax' 1.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri -`ymin' 0.5 -`ymin' 1.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri `ymax' 1.5 `ymax' 2.49, recast(area) lwidth(none) col(gs12%50) below || ///
					  scatteri -`ymin' 1.5 -`ymin' 2.49, recast(area) lwidth(none) col(gs12%50) below || ///
					  scatteri `ymax' 2.5 `ymax' 7.49, recast(area) lwidth(none) col(gs8%50) below || ///
					  scatteri -`ymin' 2.5 -`ymin' 7.49, recast(area) lwidth(none) col(gs8%50) below ///
				) ///
				yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
				yscale(r(-`ymin' `ymax')) /// y scale
				ylabel(-`ymin'(`bet')`ymax') /// y label
				graphregion(color(white)) /// white background
				xtitle("Relative time", size(large)) /// xtitle
				ytitle("Coefficient (pp)", size(large)) /// ytitle
				title("", size(vsmall) col(black)) /// no title
				text( /// text for each post period
					`height' `flag' "{bf: Flag}" `height' `punish' "{bf:Punish}" `height' `after' "{bf:After}" "{bf:flag}", size(large) ///
						orient(vertical) placement(c) ///
				) ///
				coeflabel( /// label of coefficients
					1.flagged#1.time_id = "0" 1.flagged#2.time_id = "1" 1.flagged#3.time_id = "2" ///
					1.flagged#4.time_id = "3" 1.flagged#5.time_id = "4" 1.flagged#6.time_id = "5" ///
					1.flagged#7.time_id = "6" ///
				) ///
				legend(label(8 "Full sample") label(10 "Threshold sample") size(large) region(lstyle(none))) // legend
	}

	
end

*---------------------------------------------------------------------------------------------------
**# 1. regressions: markaz red flagging #

* Figure 5

global treatments tpres satt fac math eng urdu // Panel A, B, C, D, E, F
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	
	local baseper = 4 // define base period
	
	*full regression
	eststo full: reghdfe `yvar' i.flagged##ib(`baseper').time_id [aw = avg_totalreportedteachers_mar], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)

	*threshold sample regression
	eststo rd: reghdfe `yvar' i.flagged##ib(`baseper').time_id if rd1 == 1 [aw = avg_totalreportedteachers_mar], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
		
	*set graph parameters
	
	// constant
	local xl     = 4
	local yl     = 0
	local flag   = 5
	local punish = 6 
	local after  = 9.5
	local typeg = 1
	
	// changing
	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac"){
		
		local ymax   = 6
		local ymin   = 8
		local bet    = 2
		local height = 4.2
	
	} 
	else{ // parameters for scores 
	
		local ymax   = 12
		local ymin   = 16
		local bet    = 4
		local height = 8.8
		
	}

	*plot 
	
	eventgraph `yvar' full rd `xl' `yl' `ymin' `ymax' `bet' `height' `flag' `punish' `after' `typeg'
	
	* save graph

	graph export "$analysis/figures/causal/1_`yvar'flagging_`yvar'.pdf", replace		

}

* Table 3

global treatments tpres satt fac math eng urdu 
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	
	*variables for each period
	
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
	
	*label values
	
	label define f 0 "Before" 1 "Flag"
	label values flag f
	label define p 0 "Before" 1 "Punish"
	label values punish p
	label define a 0 "Before" 1 "After flag"
	label values afterflag a
	label define t 0 "No T" 1 "T"
	label values flagged t
	
	*label
	
	label var flag "Flag"
	label var punish "Punish"
	label var afterflag "After flag"
	label var flagged "T"
	
	* full regression
	eststo full`yvar': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		[aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum `yvar' if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum `yvar' if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

	* rd regression
	eststo rd`yvar': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if rd1 == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum `yvar' if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum `yvar' if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

}

local tabs "outcomes scores" //  Panel A, B
foreach t of local tabs{
	
	if("`t'" == "outcomes"){
		
		local models fulltpres rdtpres fullsatt rdsatt fullfac rdfac
	}
	else{
		
		local models fullmath rdmath fulleng rdeng fullurdu rdurdu
	}
	
	*table
	
	estout `models' /// models
		   using "$analysis/tables/causal/1_flagging_`t'.tex", /// file
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   keep(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// coefficients to keep
		   order(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// order coefficients
		   label interact("$\times$") /// interations
		   prefoot(\midrule ) ///
		   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
		   replace // replace if it exist

}

* Figure 7

global treatments tpres satt fac math eng urdu // Panel A, B, C, D, E, F
foreach yvar of global treatments{
	
	*main data set
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	
	*size of the dip
	// value in pre-schock
	bys stack mar: egen temp = mean(mar_`yvar') if time_id <= 4 
	bys stack mar: egen premean = max(temp)
	drop temp
	// value in the shock
	bys stack mar: egen temp = mean(mar_`yvar') if time_id == 5 
	bys stack mar: egen shock = max(temp)
	drop temp
	// value in punish
	bys stack mar: egen temp = mean(mar_`yvar') if time_id == 6 
	bys stack mar: egen postmean = max(temp) 
	drop temp
	// descriptive of diff
	gen diff = postmean - premean 
	sum diff if diff < 0 , d
	// gen recovered
	gen recovered =  (diff >= `r(p75)') // closer to 0, then they would have already recovered for punish period
	// collapse at the mar level: detrended at the aggregate level
	collapse (mean) `yvar' avg_totalreportedteachers, by(stack mar time_id district flagged recovered rd1)
	bys stack mar (time_id): gen detrend = `yvar' - `yvar'[_n-1]
	
	*run month to month regression

	// month by month difference in average
	
	forvalues i = 1(1)10{
		
		local baseper = `i'
		local next    = `i'+1
		eststo full: reghdfe `yvar' i.flagged##ib(`baseper').time_id ///
		if time_id >= `i' & time_id <= `next' & ///
		recovered == 0 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack stack#district#c.time_id) vce(cluster mar)
		matrix C = e(b)[1,8] // coefficient
		matrix A = sqrt(e(V)[8,8])
		matrix g`i' = (C \ A)
		
	}
	
	matrix D = (g1[1,1], g1[2,1] \ g2[1,1], g2[2,1] \ g3[1,1], g3[2,1] \ ///
				g4[1,1], g4[2,1] \ g5[1,1], g5[2,1] \ g6[1,1], g6[2,1] \ ///
				g7[1,1], g7[2,1] \ g8[1,1], g8[2,1] \ g9[1,1], g9[2,1] \ ///
				g10[1,1], g10[2,1])
				
				
// month by month difference in average in threshold sample
				
	forvalues i = 1(1)10{
		
		local baseper = `i'
		local next    = `i'+1
		eststo rd: reghdfe `yvar' i.flagged##ib(`baseper').time_id ///
		if time_id >= `i' & time_id <= `next' & ///
		recovered == 0 & rd1 == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack stack#district#c.time_id) vce(cluster mar)
		matrix C = e(b)[1,8] // coefficient
		matrix A = sqrt(e(V)[8,8])
		matrix g`i' = (C \ A)
		
	}
	
	matrix E = (g1[1,1], g1[2,1] \ g2[1,1], g2[2,1] \ g3[1,1], g3[2,1] \ ///
				g4[1,1], g4[2,1] \ g5[1,1], g5[2,1] \ g6[1,1], g6[2,1] \ ///
				g7[1,1], g7[2,1] \ g8[1,1], g8[2,1] \ g9[1,1], g9[2,1] \ ///
				g10[1,1], g10[2,1])				
				

	local xl     = 3.5
	local yl     = 0
	local flag   = 4
	local punish = 5 
	local after  = 8
	
	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac" ){
		
		local ymax   = 6
		local ymin   = 14
		local bet    = 2
		local height = -10.2
		
	}
	else{
		
		local ymax   = 12
		local ymin   = 20
		local bet    = 4
		local height = -16.2
		
	}
	
// plot of the two set of results

	grstyle init
	grstyle set color "navy" "cranberry" 
	
	coefplot (matrix(D[,1]), se(D[,2])) ///
			(matrix(E[,1]), se(E[,2])), /// coefficients and standard error matrix
			 recast(connect) lw(vvthin) /// connected graph
			 baselevels /// include baselevel
			 omitted /// include ommited period (-1)
			 level(95) /// 95% confidence intervals
			 ciopts(recast(rcap)) /// type of confidence intervals
			 vertical /// vertical oriented graph 
			 mcol(%30) /// transparency of points
			 addplot( /// additional graph details to include
				  scatteri `ymax' 3.5 `ymax' 4.49, recast(area) lwidth(none) col(gs14%50) below || ///
				  scatteri -`ymin' 3.5 -`ymin' 4.49, recast(area) lwidth(none) col(gs14%50) below || ///
				  scatteri `ymax' 4.5 `ymax' 5.49, recast(area) lwidth(none) col(gs12%50) below || ///
				  scatteri -`ymin' 4.5 -`ymin' 5.49, recast(area) lwidth(none) col(gs12%50) below || ///
				  scatteri `ymax' 5.5 `ymax' 10.49, recast(area) lwidth(none) col(gs8%50) below || ///
				  scatteri -`ymin' 5.5 -`ymin' 10.49, recast(area) lwidth(none) col(gs8%50) below ///
			) ///
			yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
			xline(`xl', lpattern("dash") lcol("black")) /// x line for initial period
			yscale(r(-`ymin' `ymax')) /// y scale
			ylabel(-`ymin'(`bet')`ymax') /// y label
			graphregion(color(white)) /// white background
			xtitle("Relative time", size(large)) /// xtitle
			ytitle("Month on month change", size(large)) /// ytitle
			title("", size(vsmall) col(black)) /// no title
			text( /// text for each post period
				`height' `flag' "{bf: Flag}" `height' `punish' "{bf:Punish}" `height' `after' "{bf:After}" "{bf:flag}", size(large) ///
					orient(vertical) placement(c) ///
			) ///
			coeflabel( /// label of coefficients
				r1 = "-3" r2 = "-2" r3 = "-1" r4 = "0" r5 = "1" r6 = "2" r7 = "3" ///
				r8 = "4"  r9 = "5" r10 = "6" ///
			) ///
			legend(label(8 "Full sample") label(10 "Threshold sample") size(large) region(lstyle(none)))
			
	* save graph

	graph export "$analysis/figures/causal/1_`yvar'flagging_`yvar'_dip.pdf", replace		


}

	*pvalue - threshold sample Figure 7

global treatments tpres satt fac math eng urdu 
foreach yvar of global treatments{

	*main data set
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	
	*size of the dip
	// value in pre-schock
	bys stack mar: egen temp = mean(mar_`yvar') if time_id <= 4 
	bys stack mar: egen premean = max(temp)
	drop temp
	// value in the shock
	bys stack mar: egen temp = mean(mar_`yvar') if time_id == 5 
	bys stack mar: egen shock = max(temp)
	drop temp
	// value in punish
	bys stack mar: egen temp = mean(mar_`yvar') if time_id == 6 
	bys stack mar: egen postmean = max(temp) 
	drop temp
	// descriptive of diff
	gen diff = postmean - premean 
	sum diff if diff < 0 , d
	// gen recovered
	gen recovered =  (diff >= `r(p75)')
	// collapse at the mar level: detrended at the aggregate level
	collapse (mean) `yvar' avg_totalreportedteachers, by(stack mar time_id district flagged recovered rd1)
	bys stack mar (time_id): gen detrend = `yvar' - `yvar'[_n-1]
			
// month by month difference in average
	
	forvalues i = 5(1)6{
		
		preserve
		
		local baseper = `i'
		local next    = `i'+1
		keep if time_id >= `i' & time_id <= `next'
		replace time_id = 0 if time_id == `i'
		replace time_id = 1 if time_id == `next'
		
		gen group = `i'
		tempfile temp`i'
		save `temp`i''
		
		restore
				
	}	
	
	* we append both to run the mmore easily
	use `temp5', clear
	append using `temp6'
	
	*gen flagging

	gen treat = flagged * time_id // create treatment status to make the test easier
	
	/*important note! 
	
	reghdfe is not compatible to testing hypothesis across models (like in this case). Usually we 
	would be able to use test or lincom as they come from the same regression, but comparing across 
	regressions with that amount of FE might be difficult, as we have to go back to use the usual 
	reg command
	
	to address the issue we will apply FWL theorem
	
	*/
	forvalues i = 5(1)6{

	* 1. regress the diff in diff interaction on the fixed effects
		reghdfe treat if recovered == 0 & rd1 == 1 & group == `i' [aw = avg_totalreportedteachers_mar], ///
			absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar) resid
		predict rtreat`i', resid
		
	* 2. regress the outcome on the the fixed effects
		reghdfe `yvar' if recovered == 0 & rd1 == 1 & group == `i' [aw = avg_totalreportedteachers_mar], ///
			absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar) resid
		predict r`yvar'`i', resid
		
		*regress the residuals from 2 against the residuals in 1
		eststo r`i': reg r`yvar'`i' rtreat`i' if recovered == 0 & rd1 == 1 & group == `i'
			
	}
	
	
	// we see that the coefficients are the same from those of the figure, as well as the sd
	suest r5 r6, vce(cluster mar)
	// lincom and test perform a two sided test! we need to adjust to one sided
	test   [r6_mean]rtreat6 - [r5_mean]rtreat5 = 0 // wald test
	local sign_test = sign([r6_mean][rtreat6]-[r5_mean][rtreat5]) // sign of the wald test difference

	/*test 1: 
	test if the difference in the coefficients is significant, but sign matters
		- if positive: 1 - alpha
		- if negative: alpha
	*/
		
	if(`sign_test' == 1){
		display "one sided wald test for `yvar'" = 1 - normal(`sign_test'*sqrt(r(chi2)))
	}
	else if(`sign_test' == -1){
		display "one sided wald test for `yvar'" = normal(`sign_test'*sqrt(r(chi2)))
	}
	
	/*test 2: 
	test if the difference in the coefficients is positive and significant
		- if positive: 1 - alpha
		- if negative: 1 - alpha as we still want to test if the difference is positive
	*/
	
	display "one sided wald test for `yvar'" = 1 - normal(`sign_test'*sqrt(r(chi2)))

}

* Table B2

global treatments tpres satt fac // scores do not have pre datapack 3 data
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	
	*variables for each period
	
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
	
	*label values
	
	label define f 0 "Before" 1 "Flag"
	label values flag f
	label define p 0 "Before" 1 "Punish"
	label values punish p
	label define a 0 "Before" 1 "After flag"
	label values afterflag a
	label define t 0 "No T" 1 "T"
	label values flagged t
	
	*label
	
	label var flag "Flag"
	label var punish "Punish"
	label var afterflag "After flag"
	label var flagged "T"
	
	forvalues i = 1(1)3{ // regression for each datapack

		* full regression
		eststo full`yvar'`i': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
			if datapack == `i' [aw = avg_totalreportedteachers_mar], ///
			absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
		qui estadd ysumm
		qui sum `yvar' if e(sample)
		qui estadd scalar yobs `r(N)'
		qui sum `yvar' if e(sample) & relativetime == 0 
		qui estadd scalar ymeanbef `r(mean)'

		* rd regression
		eststo rd`yvar'`i': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
			if rd1 == 1 & datapack == `i' [aw = avg_totalreportedteachers_mar], ///
			absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
		qui estadd ysumm
		qui sum `yvar' if e(sample)
		qui estadd scalar yobs `r(N)'
		qui sum `yvar' if e(sample) & relativetime == 0 
		qui estadd scalar ymeanbef `r(mean)'
	}

}

local tabs "dp1 dp2 dp3" // Panel A, B, C
foreach t of local tabs{
	
	if("`t'" == "dp1"){
		
		local models fulltpres1 rdtpres1 fullsatt1 rdsatt1 fullfac1 rdfac1
	}
	else if("`t'" == "dp2"){
		
		local models fulltpres2 rdtpres2 fullsatt2 rdsatt2 fullfac2 rdfac2
	}
	else{
		
		local models fulltpres3 rdtpres3 fullsatt3 rdsatt3 fullfac3 rdfac3		
	}
	
	*table
	
	estout `models' /// models
		   using "$analysis/tables/causal/1_flagging_outcomes_`t'.tex", /// file
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   keep(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// coefficients to keep
		   order(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// order coefficients
		   label interact("$\times$") /// interations
		   prefoot(\midrule ) ///
		   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
		   replace // replace if it exist

}

*---------------------------------------------------------------------------------------------------
**# 2. regressions: markaz red flagging - short stack #

* Figure B5

global treatments tpres satt fac math eng urdu // Panel A, B, C, D, E, F
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata2_`yvar'.dta", clear
	
	local baseper = 4 // define base period
	
	*full regression
	eststo full: reghdfe `yvar' i.flagged##ib(`baseper').time_id [aw = avg_totalreportedteachers_mar], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)

	*threshold sample regression
	eststo rd: reghdfe `yvar' i.flagged##ib(`baseper').time_id if rd1 == 1 [aw = avg_totalreportedteachers_mar], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
		
	*set graph parameters
	
	// constant
	local xl     = 4
	local yl     = 0
	local flag   = 5
	local punish = 6 
	local after  = 7.5
	local typeg  = 2

	
	// changing
	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac"){
		
		local ymax   = 6
		local ymin   = 8
		local bet    = 2
		local height = 4.2
	
	} 
	else{ // parameters for scores 
	
		local ymax   = 12
		local ymin   = 16
		local bet    = 4
		local height = 8.8
		
	}

	*plot 
	
	eventgraph `yvar' full rd `xl' `yl' `ymin' `ymax' `bet' `height' `flag' `punish' `after' `typeg'
	
	* save graph

	graph export "$analysis/figures/causal/2_`yvar'flagging_`yvar'_short.pdf", replace		

}

*---------------------------------------------------------------------------------------------------
**# 3. regressions: markaz red flagging - flagging history fixed effects #

* Table B1

global treatments tpres satt fac math eng urdu 
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'_flaghist.dta", clear
	
	*variables for each period
	
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
	
	*label values
	
	label define f 0 "Before" 1 "Flag"
	label values flag f
	label define p 0 "Before" 1 "Punish"
	label values punish p
	label define a 0 "Before" 1 "After flag"
	label values afterflag a
	label define t 0 "No T" 1 "T"
	label values flagged t
	
	*label
	
	label var flag "Flag"
	label var punish "Punish"
	label var afterflag "After flag"
	label var flagged "T"
	
	* full regression
	eststo full`yvar': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		[aw = avg_totalreportedteachers_mar], ///
		absorb(`yvar'_flag_hist#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum `yvar' if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum `yvar' if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

	* rd regression
	eststo rd`yvar': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if rd1 == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(`yvar'_flag_hist#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum `yvar' if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum `yvar' if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

}

local tabs "outcomes scores" // Panel A, B
foreach t of local tabs{
	
	if("`t'" == "outcomes"){
		
		local models fulltpres rdtpres fullsatt rdsatt fullfac rdfac
	}
	else{
		
		local models fullmath rdmath fulleng rdeng fullurdu rdurdu
	}
	
	*table
	
	estout `models' /// models
		   using "$analysis/tables/causal/1_flagging_`t'_flaghist.tex", /// file
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   keep(1.flagged 1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// coefficients to keep
		   order(1.flagged 1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// order coefficients
		   label interact("$\times$") /// interations
		   prefoot(\midrule ) ///
		   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
		   replace // replace if it exist

}

*---------------------------------------------------------------------------------------------------
**# 4. regressions: markaz orange flagging #

* Figure B8

global treatments tpres satt fac math eng urdu // Panel A, B, C, D, E, F
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'_orange.dta", clear
	
	local baseper = 4 // define base period
	
	*full regression
	eststo full: reghdfe `yvar' i.flagged##ib(`baseper').time_id [aw = avg_totalreportedteachers_mar], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)

	*threshold sample regression
	eststo rd: reghdfe `yvar' i.flagged##ib(`baseper').time_id if rd1 == 1 [aw = avg_totalreportedteachers_mar], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
		
	*set graph parameters
	
	// constant
	local xl     = 4
	local yl     = 0
	local flag   = 5
	local punish = 6
	local after  = 9.5
	local typeg  = 1
	
	// changing
	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac"){
		
		local ymax   = 6
		local ymin   = 8
		local bet    = 2
		local height = 4.2
	
	} 
	else{ // parameters for scores 
	
		local ymax   = 12
		local ymin   = 16
		local bet    = 4
		local height = 8.8
		
	}

	*plot 
	
	eventgraph `yvar' full rd `xl' `yl' `ymin' `ymax' `bet' `height' `flag' `punish' `after' `typeg'
	
	* save graph

	graph export "$analysis/figures/causal/4_`yvar'flagging_`yvar'_orange.pdf", replace		

}

*---------------------------------------------------------------------------------------------------
**# 5. regressions: tehsil red flagging #

* Figure B9

global treatments  tpres satt fac math eng urdu // Panel A, B, C, D, E, F
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'_tehsil.dta", clear
	
	local baseper = 4 // define base period
	
	*full regression
	eststo full: reghdfe `yvar' i.flagged##ib(`baseper').time_id [aw = avg_totalreportedteachers_tehsil], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)

	*threshold sample regression
	eststo rd: reghdfe `yvar' i.flagged##ib(`baseper').time_id if rd1 == 1 [aw = avg_totalreportedteachers_tehsil], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
		
	*set graph parameters
	
	// constant
	local xl     = 4
	local yl     = 0
	local flag   = 5
	local punish = 6
	local after  = 9.5
	local typeg  = 1
	
	// changing
	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac"){
		
		local ymax   = 6
		local ymin   = 8
		local bet    = 2
		local height = 4.2
	
	} 
	else{ // parameters for scores 
	
		local ymax   = 20
		local ymin   = 16
		local bet    = 4
		local height = 15.8
		
	}

	*plot 
	
	eventgraph `yvar' full rd `xl' `yl' `ymin' `ymax' `bet' `height' `flag' `punish' `after' `typeg'
	
	* save graph

	graph export "$analysis/figures/causal/5_`yvar'flagging_`yvar'_tehsil.pdf", replace		

}

*---------------------------------------------------------------------------------------------------
**# 6. regressions: markaz red flagging additional post-periods #

* Figure B4

global treatments tpres satt fac // Panel A, B, C (no long enough data for scores)
foreach yvar of global treatments{
	
	forvalues i = 1(1)5{
		
		*main data set
		
		use "$analysis/processed/stacked/cc_stackdata`i'_`yvar'.dta", clear
	
		*variables for each period
		
		gen flag = (relativetime == 0)
		gen punish = (relativetime == 1)
		gen afterflag = (relativetime >= 2)

		* full regression
		eststo full`i': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
			[aw = avg_totalreportedteachers_mar], ///
			absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)

		* rd regression
		eststo rd`i': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
			if rd1 == 1 [aw = avg_totalreportedteachers_mar], ///
			absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)	

	}		
				
	*set graph parameters
	
	// constant
	local yl = 0
	
	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac"){

		local ymax = 2
		local ymin = 8
		local bet = 2
		
	}
	else{
		
		local ymax = 4
		local ymin = 16
		local bet = 4
		
	}

	*plot
	
	coefplot    full1 rd1, bylabel(1 post) /// each post t period effect
	         || full2 rd2, bylabel(2 post) ///
		     || full3 rd3, bylabel(3 post) ///
		     || full4 rd4, bylabel(4 post) ///
		     || full5 rd5, bylabel(5 post) ///
		     ||, /// finishing of model details
	         drop(_cons) /// drop constant
			 vertical /// vertical graph
			 yline(`yl', lcol("black") lpattern(dash)) ///
			 yscale(r(-`ymin' `ymax')) ylabel(-`ymin'(`bet')`ymax') ///
			 rename( ///
				1.flagged#1.flag = "Flag" 1.flagged#1.punish = "Punish" 1.flagged#1.afterflag = "After flag" ///
			 ) /// label of coefficients
			 graphregion(color(white))  /// graph region color
			 byopts(compact cols(5) graphregion(color(white))) /// compact type of graph
			 legend(order(2 "Full sample" 4 "Threshold sample") size(large) region(lstyle(none))) /// legend
			 ytitle("Coefficient (pp)", size(large)) /// title y axis
			 xlabel(, angle(vertical) labsize(large)) /// xlabel format
			 level(95) /// confidence intervals of coefficients
			 ciopts(recast(rcap)) // type of confidence intervals
	
	* save graph

	 graph export "$analysis/figures/causal/6_`yvar'flagging_`yvar'_multiple.pdf", replace		

}

*---------------------------------------------------------------------------------------------------
**# 7. regressions: markaz red flagging seasonality #

* Figure B10

global treatments tpres satt fac math urdu eng // Panel A, B, C, D, E, F
foreach yvar of global treatments{
	
	*main data set
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	
	*variables for each period
		
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
		
	* create empty matrix 
	
	matrix F = (0, 0) // full matrix
	matrix R = (0, 0) // rd matrix
	
	* run for each month
	
	levelsof stack
	foreach d in `r(levels)'{
		
		qui sum `yvar' if stack == `d' & rd == 1 // we only want stacks with rd sample
		if(`r(N)' > 50){// we run for each month if such stack has more than 50 observations
				
			display "running for flagging `yvar' on stack `d'"
				
			rename afterflag afterflag`d'
			
			*full
			eststo full`d': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag`d' ///
				if stack == `d' [aw = avg_totalreportedteachers_mar], /// 
				absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
				
			matrix coef    = e(b)[1,20] // coefficient
			matrix stder   = sqrt(e(V)[20,20]) // standard error
			matrix matcoef = (coef \ stder) // both
			
			*join with initial matrix
			matrix F = (F \ matcoef[1,1], matcoef[2,1])
			
			*threshold 
			eststo rd`d': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag`d' ///
				 if stack == `d' & rd1 == 1 [aw = avg_totalreportedteachers_mar], /// 
				absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
				
			matrix coef    = e(b)[1,20] // coefficient
			matrix stder   = sqrt(e(V)[20,20]) // standard error
			matrix matcoef = (coef \ stder) // both
			
			*join with initial matrix
			matrix R = (R \ matcoef[1,1], matcoef[2,1])

		}
	}
	
	*delete first zeros from matrix
	matrix F = F[2...,1..2]
	matrix R = R[2...,1..2]
			
	*set graph parameters
	
	// constant
	local yl = 0
	local ymax = 16
	local ymin = 16
	local bet = 4
	
	*plot 
	
	coefplot (matrix(F[,1]), se(F[,2])) ///
			(matrix(R[,1]), se(R[,2])), /// coefficients and standard error matrix
			 level(95) /// 95% confidence intervals
			 ciopts(recast(rcap)) /// type of confidence intervals
			 vertical /// vertical oriented graph 
			 mcol(%30) /// transparency of points
			yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
			yscale(r(-`ymin' `ymax')) /// y scale
			ylabel(-`ymin'(`bet')`ymax') /// y label
			graphregion(color(white)) /// white background
			xtitle("Month (event-date)", size(large)) ///
			ytitle("Coefficient (pp)", size(large)) ///
			title("", size(vsmall) col(black)) /// no title
			xlabel(, nolabels noticks) ///
			coeflabel( ///
			) ///
			legend(order(2 "Full sample" 4 "Threshold sample") size(large) region(lstyle(none)))

	* save graph

	 graph export "$analysis/figures/causal/7_`yvar'flagging`yvar'_month.pdf", replace		

}

*---------------------------------------------------------------------------------------------------
**# 8. regressions: flagging anticipation #

* Figure B11

global treatments tpres satt fac math eng urdu // Panel A, B, C, D, E, F
foreach yvar of global treatments{

	*main data set
		
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear

	*keep only measurement period

	keep if relativetime == 1

	*keep days after meeting, where markaz can react
	
	keep if day > 10

	*collapse by markaz stack
	
	collapse (mean) `yvar', by(stack mar day red_`yvar' district)

	*create (daily) underperformance 
	
	gen under = (`yvar' < red_`yvar')

	*variables for DiD
	
	gen flag_day = day if under == 1 // days for which markaz below threshold
	bysort stack mar: egen first_flag = min(flag_day) // first day flagged
	drop flag_day
	gen relativeday = day - first_flag // relative time for day
	gen never_flag = (first_flag == .) // never flagged within the month

	*create relative time indicators
	
	forvalues k = 5(-1)2 { // pre-days
		gen g_`k' = relativeday == -`k'
	}
	forvalues k = 0/10{ // post days
		gen g`k' = relativeday == `k'
	}
	gen last_flag = (relativeday == 11) // last relative day of treatment

	* we run using Sun and Abraham as it is an staggered design
	
	eventstudyinteract `yvar' g_* g0-g10, /// periods
		cohort(first_flag) control_cohort(never_flag) /// cohorts
		absorb(i.stack#i.mar i.stack#i.relativeday i.district#i.stack#c.relativeday) /// fixed effects
		vce(cluster mar) // errors

	* adjust matrix of coefficients 

	matrix C = e(b_iw)
	mata st_matrix("A",sqrt(diagonal(st_matrix("e(V_iw)"))))
	matrix C = C \ A'
	matrix list C

	*additional vector for including base period
	
	matrix g_1 = (0 \ 0)
	matrix C = (C, g_1)
	matrix list C
	
	*reorder matrix for including 0
			
	matrix D = (C[1,1], C[2,1] \ C[1,2], C[2,2] \ C[1,3], C[2,3] \ C[1,4], C[2,4] \ ///
		C[1,16], C[2,16] \ C[1,5], C[2,5] \ C[1,6], C[2,6] \ C[1,7], C[2,7] \ C[1,8], C[2,8] ///
		\ C[1,9], C[2,9] \ C[1,10], C[2,10] \ C[1,11], C[2,11] \ C[1,12], C[2,12] \ C[1,13], C[2,13] \ ///
		C[1,14], C[2,14] \ C[1,15], C[2,15])
	matrix list D
			
	*set graph parameters
	
	// constant
	local xl = 5
	local yl = 0	
	local flag = 0
	local punish = 0
	local after = 11

	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac"){

		local ymax = 20
		local ymin = 16
		local bet = 4
		local height = 18

	}
	else{
		
		local ymax = 40
		local ymin = 30
		local bet = 10
		local height = 38
		
	}

	* colorpalette
	
	grstyle init
	grstyle set color "navy" "cranberry" 

	* plot

	coefplot (matrix(D[,1]), se(D[,2])), /// coefficients and standard error matrix
			 recast(connect) lw(vvthin) /// connected graph
			 baselevels /// include baselevel
			 omitted /// include ommited period (-1)
			 level(95) /// 95% confidence intervals
			 ciopts(recast(rcap)) /// type of confidence intervals
			 vertical /// vertical oriented graph 
			 mcol(%30) /// transparency of points
			 addplot( /// additional graph details to include
				scatteri `ymax' 5.5 `ymax' 16.49, recast(area) lwidth(none) col(gs12%50) below || ///
				scatteri -`ymin' 5.5 -`ymin' 16.49, recast(area) lwidth(none) col(gs12%50) below ///
			) ///
			yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
			xline(`xl', lpattern("dash") lcol("black")) /// x line for initial period
			yscale(r(-`ymin' `ymax')) /// y scale
			ylabel(-`ymin'(`bet')`ymax') /// y label
			graphregion(color(white)) /// white background
			xtitle("Relative time", size(large)) /// xtitle
			ytitle("Coefficient (pp)", size(large)) /// ytitle
			title("", size(vsmall) col(black)) /// no title
			text( /// text for each post period
				`height' `after' "{bf:After underperformance}", size(large) ///
			) ///
			coeflabel( /// label of coefficients
				r1 = "-5" r2 = "-4" r3 = "-3" r4 = "-2" r5 = "-1" r6 = "0" r7 = "1" r8 = "2" ///
				r9 = "3"  r10 = "4" r11 = "5" r12 = "6" r13 = "7" r14 = "8" r15 = "9" r16 = "10" ///
			)			

	* save graph

	graph export "$analysis/figures/causal/8_`yvar'flagging`yvar'_anticipation.pdf", replace		

}

*---------------------------------------------------------------------------------------------------
**# 9. regressions: first time flagged and spillovers to neighbors #

*obtain neighbors dataset (origin - destiny): closest schools within the same tehsil

if(0){ // run only the FIRST time, then change to 0
	
	*balanced data
	
	/*there are schools with the same location latitude longitude, so i create a new id for the 
	latitude longitude, obtain the closest points and then merge back with the schools original ids*/
	
	use "$analysis/processed/cc_datapacks.dta", clear
	drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)

	keep emis latitude longitude // location data for each school
	drop if latitude == .
	
	* latitude and longitude are not perfect measures, then average (average location)
	bys emis: egen temp = mean(latitude)
	replace latitude = temp
	drop temp
	
	bys emis: egen temp = mean(longitude)
	replace longitude = temp
	drop temp
	
	* drop duplicates for school
	sort emis latitude longitude
	gen random = runiform()
	sort emis latitude longitude random
	bys emis: gen ord = _n
	keep if ord == 1
	drop random ord
	
	*drop duplicates for latitude and longitude
	
	keep latitude longitude // keep only geography variables

	sort latitude longitude
	gen random = runiform()
	sort latitude longitude random
	bys latitude longitude: gen ord = _n
	keep if ord == 1
	drop random ord

	gen id = _n // gen id
	
	*neighbors
	
	tempfile neigh // temporary save or origin data
	save `neigh', replace
	
	*get distances between coordinates
	
	preserve
	
	//rename variables for destiny dataset
	rename id origid
	rename latitude origlatitude
	rename longitude origlongitude
	geonear origid origlatitude origlongitude using `neigh', /// obtain neighbors
		    neighbors(id latitude longitude) ///
		    long nearcount(10) // 10 closest neighbors based on latitude and longitude
	drop if id == origid // drop if they closes is the same
	
	tempfile neighbors // temporary save or origin data
	save `neighbors', replace
	
	restore
	
	*merge distances to the data to get the longitude/latitude points
	
	//get point for origin
		
	use `neighbors', clear
	rename id tempid
	rename origid id
	merge m:1 id using `neigh'
	drop if _merge == 1
	drop _merge
	
	//change names
	
	rename id origid
	rename latitude origlatitude 
	rename longitude origlongitude
	
	// get point for destiny
	
	rename tempid  id
	merge m:1 id using  `neigh'
	keep if _merge == 3
	drop _merge

	* recover emis code for latitude longitude in the origin
	preserve 
	
	use "$analysis/processed/cc_datapacks.dta", clear
	drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
	keep tehsil emis latitude longitude // we want school whitin tehsils
	drop if latitude == .
	
	* latitude and longitude are not perfect measures, then average (average location)
	bys emis: egen temp = mean(latitude)
	replace latitude = temp
	drop temp
	
	bys emis: egen temp = mean(longitude)
	replace longitude = temp
	drop temp
	
	rename latitude origlatitude 
	rename longitude origlongitude
	rename tehsil origtehsil
	tempfile schools
	save `schools'
	
	restore
	
	joinby origlatitude origlongitude using `schools' // join neighbors coordinates with schools
	rename emis origemis

	*recover emis code for latitude longitude in the destiny
	
	preserve 
	
	use "$analysis/processed/cc_datapacks.dta", clear	
	keep tehsil emis latitude longitude
	drop if latitude == .
	
	* latitude and longitude are not perfect measures, then average (average location)
	bys emis: egen temp = mean(latitude)
	replace latitude = temp
	drop temp
	
	bys emis: egen temp = mean(longitude)
	replace longitude = temp
	drop temp

	tempfile schools
	save `schools', replace
	
	restore
	
	joinby latitude longitude using `schools'
	
	*keep only schools within the same tehsil
	
	keep if origtehsil == tehsil
	keep origemis emis tehsil
	duplicates drop origemis emis, force

	save "$analysis/processed/neighborsdata",replace
}

* Figure B2 and B3

global treatments tpres satt fac
foreach yvar of global treatments{

	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
		
	*keep unique stack schools to account for flagging in each stack
	/*identify which stack schools were flagged, and for those identify the neighbors of the same 
	tehsil
	*/
	
	keep if flagged == 1
	keep stack emis
	sort stack emis
	gen random = runiform()
	sort stack emis random
	bys stack emis: gen ord = _n
	keep if ord == 1
	drop random ord
	rename emis origemis
	
	* with this we have the set of neighbors for the flagged!
	
	joinby origemis using "$analysis/processed/neighborsdata" // the majority matched in the master
	
	* keep unique neigthbors/stack: with this identify neighbors
	sort stack emis
	gen random = runiform()
	sort stack emis random
	bys stack emis: gen ord = _n
	keep if ord == 1	
	keep emis stack
	
	* gen neighbor school indicator
	
	gen neighschool = 1
	
	*merge with stacked dataset
	
	merge 1:m stack emis using "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta"
	drop if _merge == 1
	drop _merge
	
	*generalize indicator
	
	replace neighschool = 0 if neighschool == .
	bys stack emis: egen neigh = max(neighschool)
	replace neigh = 0 if flagged == 1
	bys stack mar: egen neighmar = max(neigh)
	replace neighmar = 0 if flagged == 1

	*regressions
	
	local baseper = 4 // define base period

	* full
	eststo full: reghdfe `yvar' i.flagged##ib(`baseper').time_id i.neighmar##ib(`baseper').time_id ///
		if stack >= 10 & stack <= 11 [aw = avg_totalreportedteachers_mar], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) ///
		vce(cluster mar)

	* rd
	eststo rd: reghdfe `yvar' i.flagged##ib(`baseper').time_id i.neighmar##ib(`baseper').time_id ///
		if rd1 == 1 & stack >= 10 & stack <= 11 [aw = avg_totalreportedteachers_mar], /// 
		absorb(mar#stack time_id#stack district#stack#c.time_id) ///
		vce(cluster mar)
	
	*define parameters for  plotting
	
	// constant
	local xl = 4
	local yl = 0	
	local ymax = 16
	local ymin = 16
	local bet = 4
	local height = 8.8
	local flag   = 5
	local punish = 6 
	local after  = 9.5
	local typeg = 1
	
	*for main flagged
	
	eventgraph `yvar' full rd `xl' `yl' `ymin' `ymax' `bet' `height' `flag' `punish' `after' `typeg'
	
	* save graph

	graph export "$analysis/figures/causal/10_`yvar'flagging_`yvar'_first.pdf", replace		

	*for neighbors
	local typeg3 = 3

	
	eventgraph `yvar' full rd `xl' `yl' `ymin' `ymax' `bet' `height' `flag' `punish' `after' `typeg3'

	graph export "$analysis/figures/causal/10_`yvar'flagging_`yvar'_neigh.pdf", replace		

}

*---------------------------------------------------------------------------------------------------
**# 10. regressions: alternative difference in differences estimators #

* Figure B6: sun & abraham (2021): staggered treatment

global treatments tpres satt fac math urdu eng // Panel A, B, C, D, E, F
foreach yvar of global treatments{

	* main data: balanced dataset
	use "$analysis/processed/cc_datapacks.dta", clear
	drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
	drop if date_treat <= tm(2011m9)
	/*collapse by markaz: 
	
	note that schools are not constant across markaz, so we need to collapse at the markaz level 
	to cleanly identify the treatment dates 
	*/
	
	collapse (max) treat_`yvar'_  (mean) `yvar' totalteachers, by(mar district date_treat) // estimate markaz level regression due to memory constrains
	replace treat_`yvar'_ = 0 if date_treat <= tm(2012m1)
	
	*variables for DiD
	
	gen flag_date = date_treat if treat_`yvar'_ == 1 // date of flagging
	bysort mar: egen first_flag = min(flag_date) // for each unit, first time flagged
	drop flag_date
	gen relativemonth = date_treat - first_flag // create relative time indicator for each first flag
	gen never_flag = (first_flag == .) // create group of never flagged

	*create time dummies
	
	forvalues k = 4(-1)2 { // create pre period dummies (note -1 used as base)
		gen g_`k' = relativemonth == -`k'
	}
	forvalues k = 0/6{ // create post period dummies
		gen g`k' = relativemonth == `k'
	}

	gen last_flag = (first_flag == 693) // last month of first time flagged

	bys mar date_treat: egen w = mean(totalteachers)

	*full sample estimation of Sun & Abraham
	
	eventstudyinteract `yvar' g_* g0-g6 if first_flag != . & date_treat < 699 , /// periods
		cohort(first_flag) control_cohort(last_flag) /// cohorts
		absorb(i.mar i.date_treat i.district#c.date_treat) /// fixed effects
		vce(cluster mar) // errors
	
	* adjust matrix of coefficients 
	
	matrix C = e(b_iw)
	mata st_matrix("A",sqrt(diagonal(st_matrix("e(V_iw)"))))
	matrix C = C \ A'
	matrix list C

	*additional vector to inlcude base period
	
	matrix g_1 = (0 \ 0)
	matrix C = (C, g_1)
	matrix list C

	*reorder matrix for including 0
	
	matrix D = (C[1,1], C[2,1] \ C[1,2], C[2,2] \ C[1,3], C[2,3] \ C[1,11], C[2,11] \ ///
		C[1,4], C[2,4] \ C[1,5], C[2,5] \ C[1,6], C[2,6] \ C[1,7], C[2,7] \ C[1,8], C[2,8] ///
		\ C[1,9], C[2,9] \ C[1,10], C[2,10])
	matrix list D
	
	*set graph parameters
	
	// constant
	local xl = 4
	local yl = 0
	local flag   = 5
	local punish = 6 
	local after  = 9.5
	
	// changing
	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac"){
		
		local ymax = 6
		local ymin = 14
		local bet = 2
		local height = 4
	
	} 
	else{ // parameters for scores 
	
		local ymax = 12
		local ymin = 16
		local bet = 4
		local height = 8
		
	}

	* plot

	coefplot (matrix(D[,1]), se(D[,2])), /// coefficients and standard error matrix
			 recast(connect) lw(vvthin) /// connected graph
			 baselevels /// include baselevel
			 omitted /// include ommited period (-1)
			 level(95) /// 95% confidence intervals
			 ciopts(recast(rcap)) /// type of confidence intervals
			 vertical /// vertical oriented graph 
			 mcol(%30) /// transparency of points
			 addplot( /// additional graph details to include
				  scatteri `ymax' 4.5 `ymax' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
				  scatteri -`ymin' 4.5 -`ymin' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
				  scatteri `ymax' 5.5 `ymax' 6.49, recast(area) lwidth(none) col(gs12%50) below || ///
				  scatteri -`ymin' 5.5 -`ymin' 6.49, recast(area) lwidth(none) col(gs12%50) below || ///
				  scatteri `ymax' 6.5 `ymax' 11.49, recast(area) lwidth(none) col(gs8%50) below || ///
				  scatteri -`ymin' 6.5 -`ymin' 11.49, recast(area) lwidth(none) col(gs8%50) below ///
			) ///
			yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
			xline(`xl', lpattern("dash") lcol("black")) /// x line for initial period
			yscale(r(-`ymin' `ymax')) /// y scale
			ylabel(-`ymin'(`bet')`ymax') /// y label
			graphregion(color(white)) /// white background
			xtitle("Relative time", size(large)) /// xtitle
			ytitle("Coefficient (pp)", size(large)) /// ytitle
			title("", size(vsmall) col(black)) /// no title
			text( /// text for each post period
				`height' `flag' "{bf: Flag}" `height' `punish' "{bf:Punish}" `height' `after' "{bf:After}" "{bf:flag}", size(large) ///
					orient(vertical) placement(c) ///
			) ///
			coeflabel( /// label of coefficients
				r1 = "-4" r2 = "-3" r3 = "-2" r4 = "-1" r5 = "0" r6 = "1" r7 = "2" r8 = "3" ///
				r9 = "4"  r10 = "5" r11 = "6" ///
			)			

	* save graph

	graph export "$analysis/figures/causal/11_`yvar'flagging_didsun.pdf", replace		
		
}

*Figure B7: dechaisemartin & haoulfoullie (2021): treatment on/off

global treatments tpres satt fac math urdu eng // Panel A, B, C, D, E, F
foreach yvar of global treatments{

	* main data: balanced dataset
	
	use "$analysis/processed/cc_datapacks.dta", clear
	drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
	
	*adjust treatment to pre shock
	
	bys mar emis (date_treat): gen treat = treat_`yvar'_[_n+1]
	
	* regression
	bys mar date_treat: egen w = mean(totalteachers)
	
	tempfile estimates_dechais
	
	did_multiplegt `yvar' mar date_treat treat_`yvar'_, /// basic model
					robust_dynamic dynamic(4) placebo(3) long  ///
					breps(5) /// bootstrap replication
					seed(12345) /// set seed for replication
					trends_lin(district) /// linear trends
					weight(w) /// weight by size in terms of teachers 
					cluster(mar) save(`estimates_dechais') // cluster var
					
	use `estimates_dechais', clear

	
	*parameters
	
	local xl     = -0.5
	local yl     = 0
	local flag   = 0
	local punish = 1 
	local after  = 3.2
	
	// changing
	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac"){
		
		local ymax   = 8
		local ymin   = 10
		local bet    = 2
		local height = 5.5
	}
	else{
		
		local ymax   = 12
		local ymin   = 16
		local bet    = 4
		local height = 8.8
		
	}
	
	twoway (scatteri `ymax' -0.5 `ymax' 0.49, recast(area) lwidth(none) col(gs14%50)) ///
		   (scatteri -`ymin' -0.5 -`ymin' 0.49, recast(area) lwidth(none) col(gs14%50)) ///
		   (scatteri `ymax' 0.5 `ymax' 1.49, recast(area) lwidth(none) col(gs12%50)) ///
		   (scatteri -`ymin' 0.5 -`ymin' 1.49, recast(area) lwidth(none) col(gs12%50)) ///
		   (scatteri `ymax' 1.5 `ymax' 4.49, recast(area) lwidth(none) col(gs8%50)) ///
		   (scatteri -`ymin' 1.5 -`ymin' 4.49, recast(area) lwidth(none) col(gs8%50)) ///
			(connect treatment_effect time_to_treatment, mcol(navy%30) lwidth(vvthin)) /// coefficient 
		   (rcap treatment_effect_upper_95CI treatment_effect_lower_95CI time_to_treatment, lcol(navy)), /// 
			yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
			xline(`xl', lpattern("dash") lcol("black")) /// x line for initial period
			yscale(r(-`ymin' `ymax')) /// y scale
			ylabel(-`ymin'(`bet')`ymax') /// y label
			xlabel(-4(1)4) ///
			graphregion(color(white)) /// white background
			xtitle("Relative time", size(large)) /// xtitle
			ytitle("Coefficient (pp)", size(large)) /// ytitle
			title("", size(vsmall) col(black)) /// no title
			text( /// text for each post period
				`height' `flag' "{bf: Flag}" `height' `punish' "{bf:Punish}" `height' `after' "{bf:After}" "{bf:flag}", size(large) ///
					orient(vertical) placement(c) ///
			) ///
		    legend(off)
		
	* save graph

	graph export "$analysis/figures/causal/11_`yvar'flagging_dechais.pdf", replace		

}

*---------------------------------------------------------------------------------------------------
**# 11. regressions: bayesian approach: cumulative effect #

* Figure B12

global treatments tpres satt fac // Panel A, B, C (no scores as have small number of stacks / time series)
foreach yvar of global treatments{

	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	
	*variables for each period
		
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
	
	*id for dates
	egen id_month = group(stack) 
	*drop if id_month == 31 | id_month == 32 | id_month == 33
	
	forvalues i = 1(1)45{
		
		display "running regressions for `yvar' flagging on `i' month"

		* full regression
		eststo full`i': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
			if id_month <= `i' [aw = avg_totalreportedteachers_mar], ///
			absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)

		
		* rd regression
		eststo rd`i': reghdfe `yvar' i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
			if rd1 == 1 & id_month <= `i' [aw = avg_totalreportedteachers_mar], ///
			absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)	
	
	}
		
	// constant
	local yl = 0	
	local ymax = 2
	local ymin = 6
	local bet = 2

	*graph
	
	* colorpalette
	
	grstyle init
	grstyle set color "navy" "cranberry" 
	
	* plot
	
	*full sample
	coefplot full1  || full2  || full3  || full4  || full5  || full6  || full7  || full8  || ///
			 full9  || full10 || full11 || full12 || full13 || full14 || full15 || full16 || ///
			 full17 || full18 || full19 || full20 || full21 || full22 || full23 || full24 || ///
			 full25 || full26 || full27 || full28 || full29 || full30 || full31 || full32 || ///
			 full33 || full34 || full35 || full36 || full37 || full38 || full39 || full40 || ///
			 full41 || full42 || full43 || full44 || full45, ///
		keep(1.flagged#1.afterflag) ///
		bycoefs ///
		ciopts(recast(rarea) col("navy%30")) ///
		vertical ///
		msize(small) ///
		mcolor("navy") ///
		level(95) ///
		yscale(r(-`ymin' `ymax')) ///
		ylabel(-`ymin'(`bet')`ymax') ///
		yline(`yl', lcol("black") lpattern(dash)) ///
		graphregion(color(white)) grid(none) ///
		xlabel("1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45", ///
			labsize(vsmall)) ///
		l1title("Full sample", size(large)) ///
		xtitle("Month (acumulated)", size(large)) ///
		ytitle("Coefficient (pp)", size(large)) ///
		saving(full, replace)

	*rd
	coefplot rd1  || rd2  || rd3  || rd4  || rd5  || rd6  || rd7  || rd8  || ///
			 rd9  || rd10 || rd11 || rd12 || rd13 || rd14 || rd15 || rd16 || ///
			 rd17 || rd18 || rd19 || rd20 || rd21 || rd22 || rd23 || rd24 || ///
			 rd25 || rd26 || rd27 || rd28 || rd29 || rd30 || rd31 || rd32 || ///
			 rd33 || rd34 || rd35 || rd36 || rd37 || rd38 || rd39 || rd40 || ///
			 rd41 || rd42 || rd43 || rd44 || rd45, ///
		keep(1.flagged#1.afterflag) ///
		bycoefs ///
		ciopts(recast(rarea) col("cranberry%30")) ///
		vertical ///
		msize(small) ///
		mcolor("cranberry") ///
		level(95) ///
		yscale(r(-`ymin' `ymax')) ///
		ylabel(-`ymin'(`bet')`ymax') ///
		ytick(-`ymin'(`bet')`ymax') ///
		yline(`yl', lcol("black") lpattern(dash)) ///
		graphregion(color(white)) grid(none) ///
		xlabel("1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45", ///
			labsize(vsmall)) ///
		l1title("Threshold sample", size(large)) ///
		xtitle("Month (acumulated)" , size(large)) ///
		ytitle("Coefficient (pp)" , size(large)) ///
		saving(rd, replace)
			
	*saving full graph
	
	graph combine full.gph rd.gph, graphregion(color(white)) rows(2) 
		
	graph export "$analysis/figures/causal/12_`yvar'bayesianmain.pdf", replace
			
}

*---------------------------------------------------------------------------------------------------
**# 12. regressions: effect on effort #

* Table B5

global treatments tpres satt fac math eng urdu 
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	
	*variables for each period
	
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
	
	*outcome
	sort stack mar emis time_id advis
	gen random = runiform()
	sort stack mar emis time_id advis random
	
	bys stack mar emis (time_id advis random): gen ladminvisit = advis[_n+1] // probability of visit by aeo
	
	*label values
	
	label define f 0 "Before" 1 "Flag"
	label values flag f
	label define p 0 "Before" 1 "Punish"
	label values punish p
	label define a 0 "Before" 1 "After flag"
	label values afterflag a
	label define t 0 "No T" 1 "T"
	label values flagged t
	
	*label
	
	label var flag "Flag"
	label var punish "Punish"
	label var afterflag "After flag"
	label var flagged "T"
	
	* full regression
	eststo full`yvar': reghdfe ladminvisit i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		[aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum ladminvisit if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum ladminvisit if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

	* rd regression
	eststo rd`yvar': reghdfe ladminvisit i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if rd1 == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum ladminvisit if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum ladminvisit if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

}

local tabs "outcomes scores" // Panel A, B
foreach t of local tabs{
	
	if("`t'" == "outcomes"){
		
		local models fulltpres rdtpres fullsatt rdsatt fullfac rdfac
	}
	else{
		
		local models fullmath rdmath fulleng rdeng fullurdu rdurdu
	}
	
	*table
	
	estout `models' /// models
		   using "$analysis/tables/causal/4_flagging_`t'_admin.tex", /// file
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   keep(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// coefficients to keep
		   order(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// order coefficients
		   label interact("$\times$") /// interations
		   prefoot(\midrule ) ///
		   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
		   replace // replace if it exist

}

* Table B8

global treatments tpres satt fac math eng urdu 
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	
	*variables for each period
	
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
		
	*label values
	
	label define f 0 "Before" 1 "Flag"
	label values flag f
	label define p 0 "Before" 1 "Punish"
	label values punish p
	label define a 0 "Before" 1 "After flag"
	label values afterflag a
	label define t 0 "No T" 1 "T"
	label values flagged t
	
	*label
	
	label var flag "Flag"
	label var punish "Punish"
	label var afterflag "After flag"
	label var flagged "T"
	
	* full regression
	eststo full`yvar': reghdfe change_teacher i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		[aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum change_teacher if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum change_teacher if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

	* rd regression
	eststo rd`yvar': reghdfe change_teacher i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if rd1 == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum change_teacher if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum change_teacher if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

}

local tabs "outcomes scores" // Panel A, B
foreach t of local tabs{
	
	if("`t'" == "outcomes"){
		
		local models fulltpres rdtpres fullsatt rdsatt fullfac rdfac
	}
	else{
		
		local models fullmath rdmath fulleng rdeng fullurdu rdurdu
	}
	
	*table
	
	estout `models' /// models
		   using "$analysis/tables/causal/6_flagging_`t'_headteacher.tex", /// file
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   keep(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// coefficients to keep
		   order(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// order coefficients
		   label interact("$\times$") /// interations
		   prefoot(\midrule ) ///
		   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
		   replace // replace if it exist

}

*---------------------------------------------------------------------------------------------------
**# 13. regressions: budget #

* join budget data with main data

if(1){
	
	*run to fix financing data
	use "$analysis/processed/cc_datapacks.dta", clear
	drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
	
	*need to create dataset with thresholds for each variable/date
	local vars tpres satt fac
	foreach yvar of local vars{
		
		preserve
		 
		use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
		keep if relativetime == 0
		keep if rd1 == 1
		keep mar date_treat rd1 bw
		sort mar date_treat rd1 bw
		gen random = runiform()
		sort mar date_treat rd1 bw random
		bys mar date_treat (random): gen ord = _n
		keep if ord == 1
		drop random ord
		tempfile temp
		save `temp'
		
		restore
		
		merge m:1 mar date_treat using `temp'
		drop _merge
		rename rd1 rd`yvar'
		rename bw1 bw`yvar'

	}

	*collapse at the school mar date level the number of times flagged for each variable
	local vars tpres satt fac
	foreach v of local vars{
		
		gen `v'_thres = 1 if (treat_`v'_ == 1 & rd`v' == 1) // if flagged by being in the threshold sample
		
	}
	
	*gen year census
	gen     yearcensus = 2012 if (date_treat >= tm(2011m7) & date_treat <= tm(2012m6))
	replace yearcensus = 2013 if (date_treat >= tm(2012m7) & date_treat <= tm(2013m6))
	replace yearcensus = 2014 if (date_treat >= tm(2013m7) & date_treat <= tm(2014m6))
	replace yearcensus = 2015 if (date_treat >= tm(2014m7) & date_treat <= tm(2015m6))
	replace yearcensus = 2016 if (date_treat >= tm(2015m7) & date_treat <= tm(2016m6))
	replace yearcensus = 2017 if (date_treat >= tm(2016m7) & date_treat <= tm(2017m6))
	
	*collapse 	
	collapse (sum) treat_tpres_ treat_satt_ treat_fac_  ///
				   tpres_thres satt_thres fac_thres /// 
				   rdtpres rdsatt rdfac ///
				   , by(emis mar district yearcensus)

	drop if yearcensus == .
	
	*we merge the balanced dataset with a supporting file having the finance information

	merge m:1 emis yearcensus using "$data/cc_budget.dta"
	keep if _merge == 3
	drop _merge
	
	*gen unique school code (given that some schools might change from mar)

	egen id = group(emis mar)
	
	* fix duplicates
	local vars treat_tpres_ treat_satt_ treat_fac_ rdtpres rdsatt rdfac
	foreach v of local vars{
		
		bys id year: egen x = nvals(`v')
		bys id year: egen temp = max(`v')
		replace `v' = temp if x > 1
		drop x temp
			
	}
	
	* multiple districts
	bys id year: egen numdis = nvals(district)
	bys id year district: egen countdis = count(district)
	bys id year: egen maxdis = max(countdis)
	gen tempdis = district if maxdis == countdis & numdis != 1
	bys id year: egen newdis = max(tempdis) if numdis != 1
	replace district = newdis if numdis != 1
	drop numdis countdis maxdis tempdis newdis

	* drop duplicates
	sort id year
	gen random = runiform()
	sort id year random
	bys id year (random): gen ord = _n 
	keep if ord == 1
	
	*schools might have multiple markaz in a year: assign that of the most prevalent flagging
	bys emis year: egen nummar = nvals(mar)
	gen totalflag = treat_tpres_ + treat_satt_ + treat_fac_
	bys emis year: egen maxflag = max(totalflag)
	keep if totalflag == maxflag
	drop nummar totalflag maxflag

	*schools might have multiple markaz in a year: assign that of the most prevalent markaz
	
	bys emis year: egen nummar = nvals(mar)
    bys mar year: egen totalmar = count(mar)
	bys emis year: egen maxmar = max(totalmar)
	keep if maxmar == totalmar
	drop nummar totalmar maxmar 
	
	*schools might (still) have multiple markaz in a year: assign that of the higher coded markaz
	
	bys emis year: egen nummar = nvals(mar)
	bys mar year : egen totalmar = count(mar)
	bys emis year: egen maxmar = max(totalmar)
	keep if maxmar == totalmar
	drop nummar totalmar maxmar 
	
	bys emis year: egen nummar = nvals(mar)
	bys emis year: egen maxmarcode = max(mar)	
	keep if mar == maxmarcode
	drop nummar
	
	*adjust funds data

	xtset emis year
	sort emis year

	//lag one period to explore num of flags in future budget
	gen govfunds_lag = F.govfunds
	gen nongovfunds_lag = F.nongovfunds
	gen expenses_lag = F.exp 
	
	*gen total
	egen total_funds_lag = rowtotal(govfunds_lag nongovfunds_lag), m

	*drop last year
	drop if year == 2017

	*convert outcomes to logs

	local vals govfunds_lag nongovfunds_lag expenses_lag total_funds_lag
	foreach v of local vals{
			
		gen l`v' = log(`v'+1) // to account for reported 0
		replace l`v' = . if `v' == .
	}
	
	replace lexpenses_lag = . if lgovfunds_lag == .
	drop if lexpenses_lag == . & lgovfunds_lag == .
	
	}

*label variables 

label var treat_tpres_ "Num times flagged (teacher presence)"
label var treat_satt_  "Num times flagged (student attendance)"
label var treat_fac_   "Num times flagged (functional facilities)"

* Table B7 Panel A

local outcomes total_funds_lag govfunds_lag nongovfunds_lag expenses_lag
local treatments tpres satt fac
foreach t of local treatments{
			
		foreach o of local outcomes{
			
			eststo r`o': reghdfe l`o' treat_`t'_ , /// 
				absorb(mar year district#c.year) vce(cluster mar)
			qui estadd ysumm
			qui sum `o' if e(sample)
			qui estadd scalar yobs `r(N)'
			qui sum `o'  
			qui estadd scalar ymeanbef `r(mean)'

		}
		
		estout rtotal_funds_lag rgovfunds_lag rnongovfunds_lag rexpenses_lag using "$analysis/tables/causal/17_budget_`t'.tex", ///
			   style(tex) /// style of exporting
			   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
			   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
			   keep(treat_`t'_) ///
			   mlabels(none) /// no label for models
			   collabels(none) /// no label for columns
			   label ///
			   prefoot(\midrule ) ///
			   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
				labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
			   replace // replace if it exist
		
	}

* Table B7 Panel B

local outcomes total_funds_lag govfunds_lag nongovfunds_lag expenses_lag
local treatments tpres satt fac
foreach t of local treatments{
			
	foreach o of local outcomes{
		
		display "outcome `o'"
	
		eststo r`o': ivreghdfe l`o'  (treat_`t'_ = `t'_thres), /// keep only schools within threshold ... 
			ffirst absorb(mar year district#c.year) cluster(mar)

		qui estadd ysumm
		qui sum `o' if e(sample)
		qui estadd scalar yobs `r(N)'
		qui sum `o'  
		qui estadd scalar ymeanbef `r(mean)'
		qui estadd scalar ftest e(first)[4,1]

	}
	
	estout rtotal_funds_lag rgovfunds_lag rnongovfunds_lag rexpenses_lag using "$analysis/tables/causal/17_budget_`t'_iv.tex", ///
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   label ///
		   prefoot(\midrule ) ///
		   stats(N ftest ymeanbef, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "F-stat" "Mean Dep. Var. before")) /// stats to keep
		   replace // replace if it exist
	
}

*---------------------------------------------------------------------------------------------------
**# 14. regressions: political alignment #

* Table B4 

global treatments tpres satt fac treat_tpres_ treat_satt_ treat_fac_ // outcomes and probability of flagging
foreach yvar of global treatments{
	
	*run to fix financing data
	
	use "$analysis/processed/cc_datapacks.dta", clear
	drop if wing == 3 // we do not consider secondary schools (not subject to monitoring)
	
	*merge wit data from Callen & Gulzar (2020)
	
	merge m:1 emis using "$data/political_alignment"
	keep if _merge == 3
	
	*keep months after and before the 2013 elections
	
	keep if date_treat >= tm(2012m5)  & date_treat <= tm(2014m5)

	*gen time indicator
	
	egen time_id = group(date_treat) // elections happen in 10
	gen relativetime = time_id - 10

	*aggregate at the mar level

	//need to create an indicator when the markaz is not fully aligned
	
	bys mar: egen tempmax = max(samewin)
	bys mar: egen tempmin = min(samewin)
	gen align = 1 if (tempmax == tempmin) & tempmax == 1
	replace align = 0 if (tempmax == tempmin) & tempmax == 0
	replace align = 2 if align == .

	*collapse at the mar time level
	
	if("`yvar'" == "tpres" | "`yvar'" == "satt" | "`yvar'" == "fac" ){
		
		collapse (max) align (mean) `yvar' votemargin, ///
			by(time_id relativetime mar date_treat district)
		
	}
	else{
		
		collapse (max) `yvar' align (mean) votemargin, ///
			by(time_id relativetime mar date_treat district)
	}
	*gen after election indicator
	
	gen after = (time_id > 9)

	*rdbandwidth

	//optimal bandwidth selection for close elections 
	
	rdbwselect `yvar' votemargin if relativetime == 0, ///
		c(0) p(1) q(2) kernel(triangular) bwselect(mserd) vce(cluster mar) 
	
	//create bandwidth variable
	
	gen bwclose = `e(b_mserd)'
	
	//gen close elections indicator
	
	gen closelections = (votemargin < bwclose & votemargin >= -bwclose)
	
	*label values
	
	label define a 0 "Before" 1 "After elections"
	label values after a
	label define al 0 "Not aligned" 1 "Fully aligned" 2 "Not fully aligned"
	label values align al
	
	*regressions
	eststo full`yvar': reghdfe `yvar' ib(0).align##i.after, ///
		absorb(mar time_id distric#c.time_id) vce(cluster mar)
		qui estadd ysumm
		qui sum `yvar' if e(sample)
		qui estadd scalar yobs `r(N)'
		qui sum `yvar' if e(sample) & relativetime == 0 
		qui estadd scalar ymeanbef `r(mean)'

	eststo rd`yvar': reghdfe `yvar' ib(0).align##i.after if closelection == 1, ///
		absorb(mar time_id distric#c.time_id) vce(cluster mar)
		qui estadd ysumm
		qui sum `yvar' if e(sample)
		qui estadd scalar yobs `r(N)'
		qui sum `yvar' if e(sample) & relativetime == 0 
		qui estadd scalar ymeanbef `r(mean)'
	
}

local tabs "prob outcomes" // Panel A, B
foreach t of local tabs{
	
	if("`t'" == "outcomes"){
		
		local models fulltpres rdtpres fullsatt rdsatt fullfac rdfac
	}
	else{
		
		local models fulltreat_tpres_ rdtreat_tpres_ fulltreat_satt_ rdtreat_satt_ fulltreat_fac_ rdtreat_fac_
	}
	
	*table
	
	estout `models' /// models
		   using "$analysis/tables/causal/4_voterclose_`t'.tex", /// file
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   keep(1.align#1.after  2.align#1.after) /// coefficients to keep
		   order(1.align#1.after  2.align#1.after) /// order coefficients
		   label interact("$\times$") /// interations
		   prefoot(\midrule ) ///
		   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
		   replace // replace if it exist

}

*---------------------------------------------------------------------------------------------------
**# 15. regressions: district ranking #

* Figure 6

local datasets "bottom top" // Panel (A, B, C), (D E F)
foreach d of local datasets{
	
	*read dataset
	
	use "$analysis/processed/stacked/cc_stackdata_`d'_district.dta", clear
	
	*run for each outcomes: no scores as they go only after 2015 (last time we have data on rankings)
	
	local outcomes tpres satt fac
	foreach yvar of local outcomes{
		
		* full

		eststo full: reghdfe `yvar' i.flagged##ib(4).time_id ///
			[aw = avg_totalreportedteachers_dis], /// 
			absorb(district#stack time_id#stack) vce(cluster district) // district and time fe, district errors

		* rd
		eststo rd: reghdfe `yvar' i.flagged##ib(4).time_id ///
			if rd_`d' == 1 [aw = avg_totalreportedteachers_dis], /// 
			absorb(district#stack time_id#stack) vce(cluster district)
			
		*set graph parameters
	
		// constant
		local xl     = 4
		local yl     = 0
		local flag   = 5
		local after  = 7
		local ymax   = 6
		local ymin   = 6
		local bet    = 2
		local height = 4.8
		
		* colorpalette
	
		grstyle init
		grstyle set color "navy" "cranberry" 

		*graph 
		
		coefplot (full, offset(0.1)) /// full sample coefficients
				 (rd, offset(-0.1)), /// threshold sample coefficient
				 recast(connect) lw(vvthin) /// connected graph
				 baselevels /// include baselevel
				 omitted /// include ommited period (-1)
				 level(95) /// 95% confidence intervals
				 ciopts(recast(rcap)) /// type of confidence intervals
				 vertical /// vertical oriented graph 
				 mcol(%30) /// transparency of points
				 keep( /// interactions to keep in the graph
					  1.flagged#1.time_id 1.flagged#2.time_id 1.flagged#3.time_id ///
					  1.flagged#4.time_id 1.flagged#5.time_id 1.flagged#6.time_id ///
					  1.flagged#7.time_id 1.flagged#8.time_id ///
				) /// 
				addplot( /// additional graph details to include
					  scatteri `ymax' 4.5 `ymax' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri -`ymin' 4.5 -`ymin' 5.49, recast(area) lwidth(none) col(gs14%50) below || ///
					  scatteri `ymax' 5.5 `ymax' 8.49, recast(area) lwidth(none) col(gs12%50) below || ///
					  scatteri -`ymin' 5.5 -`ymin' 8.49, recast(area) lwidth(none) col(gs12%50) below ///
				) ///
				yline(`yl', lcol("black") lpattern(dash)) /// y line for 0
				xline(`xl', lpattern("dash") lcol("black")) /// x line for initial period
				yscale(r(-`ymin' `ymax')) /// y scale
				ylabel(-`ymin'(`bet')`ymax') /// y label
				graphregion(color(white)) /// white background
				xtitle("Relative time", size(large)) /// xtitle
				ytitle("Coefficient (pp)", size(large)) /// ytitle
				title("", size(vsmall) col(black)) /// no title
				text( /// text for each post period
					`height' `flag' "{bf: Meeting}" `height' `after' "{bf:After meeting}", size(large) ///
						orient(horizontal) placement(c) ///
				) ///
				coeflabel( /// label of coefficients
					1.flagged#1.time_id = "-4" 1.flagged#2.time_id = "-3" ///
					1.flagged#3.time_id = "-2" 1.flagged#4.time_id = "-1" ///
					1.flagged#5.time_id = "0"  1.flagged#6.time_id =  "1" ///
					1.flagged#7.time_id = "2"  1.flagged#8.time_id = "3" ///
				) ///
				legend(label(6 "Full sample") label(8 "Threshold sample") size(large) region(lstyle(none))) // legend

		* save graph

		 graph export "$analysis/figures/causal/3_`yvar'`d'_district.pdf", replace		

	}
		
}

* Table 4

local datasets "bottom top" // Panel A, B
foreach d of local datasets{
	
	*read dataset
	
	use "$analysis/processed/stacked/cc_stackdata_`d'_district.dta", clear
	
	*time indicators
	gen meeting = (time_id == 5)
	gen aftermeeting = (time_id > 5)
	
	*label values
	label define meet 0 "Before" 1 "Meeting"
	label values meeting meet
	label define aftermeet 0 "Before" 1 "After meeting"
	label values aftermeeting aftermeet
	
	if("`d'" == "bottom"){
		
		label define f 0 "Before" 1 "Bottom"
		label values flagged f
	}
	else{
		
		label define f 0 "Before" 1 "Top"
		label values flagged f
		
	}
		
	*run for each outcomes: no scores as they go only after 2015 (last time we have data on rankings)
	
	local outcomes tpres satt fac
	foreach yvar of local outcomes{

		* full
		eststo full`yvar': reghdfe `yvar' i.flagged##i.meeting i.flagged##i.aftermeeting ///
			[aw = avg_totalreportedteachers_dis], /// 
			absorb(district#stack time_id#stack) vce(cluster district)
			
			qui estadd ysumm
			qui sum `yvar' if e(sample)
			qui estadd scalar yobs `r(N)'
			qui sum `yvar' if e(sample) & relativetime == 0 
			qui estadd scalar ymeanbef `r(mean)'


		* rd
		eststo rd`yvar': reghdfe `yvar' i.flagged##i.meeting i.flagged##i.aftermeeting ///
			if rd_`d' == 1 [aw = avg_totalreportedteachers_dis], /// 
			absorb(district#stack time_id#stack) vce(cluster district)
		
			qui estadd ysumm
			qui sum `yvar' if e(sample)
			qui estadd scalar yobs `r(N)'
			qui sum `yvar' if e(sample) & relativetime == 0 
			qui estadd scalar ymeanbef `r(mean)'
	}
	
	estout fulltpres rdtpres fullsatt rdsatt fullfac rdfac /// models
	   using "$analysis/tables/causal/3_ranking_`d'.tex", /// file
	   style(tex) /// style of exporting
	   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
	   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
	   mlabels(none) /// no label for models
	   collabels(none) /// no label for columns
	   keep(1.flagged#1.meeting 1.flagged#1.aftermeeting) /// coefficients to keep
	   order(1.flagged#1.meeting 1.flagged#1.aftermeeting) /// order coefficients
	   label interact("$\times$") /// interations
	   prefoot(\midrule ) ///
	   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
		labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
	   replace // replace if it exist
		
}

* Table B3 

local datasets "bottom top" // Panel A, B
foreach d of local datasets{

	*read dataset
	
	use "$analysis/processed/stacked/cc_stackdata_`d'_district.dta", clear

	*flagging status for each variable
	local outcomes tpres satt fac
	foreach yvar of local outcomes{
		
		gen tempflag = (mar_`yvar' < red_`yvar' & relativetime == 0)
		bys stack mar: egen flag`yvar' = max(tempflag)
		drop tempflag
		
	}

	*time indicators
	gen meeting = (time_id == 5)
	gen aftermeeting = (time_id > 5)
	
	*label values
	label define meet 0 "Before" 1 "Meeting"
	label values meeting meet
	label define aftermeet 0 "Before" 1 "After meeting"
	label values aftermeeting aftermeet

	if("`d'" == "bottom"){
		
		label define f 0 "Before" 1 "Bottom"
		label values flagged f
	}
	else{
		
		label define f 0 "Before" 1 "Top"
		label values flagged f
		
	}
	
	label define mar 0 "No flag" 1 "Flag"

	*regression fo reach outcome
	
	local outcomes tpres satt fac
	foreach yvar of local outcomes{
	
		preserve
		
		rename flag`yvar' flag
		label values flag mar
		
		* full
		eststo full`yvar': reghdfe `yvar' i.flagged##i.flag##i.meeting i.flagged##i.flag##i.aftermeeting ///
			[aw = avg_totalreportedteachers_dis], /// 
			absorb(district#stack time_id#stack) vce(cluster district)
			
			qui estadd ysumm
			qui sum `yvar' if e(sample)
			qui estadd scalar yobs `r(N)'
			qui sum `yvar' if e(sample) & relativetime == 0 
			qui estadd scalar ymeanbef `r(mean)'

		* rd
		eststo rd`yvar': reghdfe `yvar' i.flagged##i.flag##i.meeting i.flagged##i.flag##i.aftermeeting ///
			if rd_`d' == 1 [aw = avg_totalreportedteachers_dis], /// 
			absorb(district#stack time_id#stack) ///
			vce(cluster district)
	
			qui estadd ysumm
			qui sum `yvar' if e(sample)
			qui estadd scalar yobs `r(N)'
			qui sum `yvar' if e(sample) & relativetime == 0 
			qui estadd scalar ymeanbef `r(mean)'

		restore
	}

	* tables
	
	estout fulltpres rdtpres fullsatt rdsatt fullfac rdfac /// models
	   using "$analysis/tables/causal/3_ranking_`d'flag.tex", /// file
	   style(tex) /// style of exporting
	   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
	   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
	   mlabels(none) /// no label for models
	   collabels(none) /// no label for columns
	keep( ///
		1.flagged#1.flag#1.meeting 1.flagged#1.meeting 1.flag#1.meeting ///
		1.flagged#1.flag#1.aftermeeting 1.flagged#1.aftermeeting 1.flag#1.aftermeeting ///
		) ///
	order( ///
		1.flagged#1.flag#1.meeting 1.flagged#1.meeting 1.flag#1.meeting ///
		1.flagged#1.flag#1.aftermeeting 1.flagged#1.aftermeeting 1.flag#1.aftermeeting ///
		) ///
	   label interact("$\times$") /// interations
	   prefoot(\midrule ) ///
	   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
		labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
	   replace // replace if it exist

}

*---------------------------------------------------------------------------------------------------
**# 16. regressions: labor market #

* Table B9
local datasets bottom top // Table B9
foreach d of local datasets{
	
	use "$analysis/processed/stacked/cc_stackdata_`d'_district.dta", clear

	/*drop duplicates as district outcomes are constant across markaz and schools, so we only keep 
	one observation per district/date/stack*/
	
	*duplicates drop district stack date_treat rd_`d' flagged avg_totalreportedteachers_dis ///
		*time_id relativetime, force
		
	collapse (mean) avg_totalreportedteachers_dis (max) flagged, by (district stack date_treat rd_`d'  ///
		time_id relativetime)

	*keep desired variables
	
	keep district stack date_treat rd_`d' flagged avg_totalreportedteachers_dis time_id relativetime
	
	*create meeting variables
	
	gen meeting = (time_id == 5)
	gen aftermeeting = (time_id > 5)
	decode district, gen(district_string)

	*here I clean a supporting dataset with the older and current ranking of district comissioners
	
	preserve 
	
	import delimited "$data\officers_punjab_pak.csv", clear 
	
	*drop unnecesary variables
	
	drop _merge dc 
	drop country province officer_federal
	rename from from1
	rename to from2
	drop if district == ""
	drop if officer_id == .
	
	*duplicates drop	
	bys district officer_id from1 from2: egen x = count(officer_id)
	drop if x == 2 & officer_id == 52 // duplicate district officer with one ranking empty
	drop x
	
	sort district officer_id from1 from2
	gen random = runiform()
	sort district officer_id from1 from2 random
	bys district officer_id from1 from2: gen ord = _n
	keep if ord == 1
	drop random ord
		
	*gen id for district to reshape
	
	bys district: gen temp_id = _n
	reshape long from, i(district temp_id) j(date)
	
	*fix dates
	
	gen year = substr(from, 1, 4)
	destring year, replace
	gen month = substr(from, 6, 7)
	destring month, replace
	gen date_treat = ym(year, month)
	format date_treat %tm
	
	*standardize district codes to merge
	
	encode district, gen(district_code)
	drop district month year temp_id
	
	/*for each district date I need to assign only one district officer - in most of the cases 
	there are at least two district officers per date */	
	bys district date_treat (officer_id rank_designation): gen temp = _n
	bys district date_treat: gen temp2 = _N // number of district officers per date
	
	* keep the one with the highest rank
	bys district date_treat (officer_id rank_designation): egen tempmaxrank = min(rank_designation) if temp2 > 1
	drop if temp2 > 1 & rank_designation != tempmaxrank
	drop temp*
	bys district date_treat (officer_id rank_designation): gen temp = _n
	bys district date_treat: gen temp2 = _N // number of district officers per date
	
	/*now, there are some still repeated with the same rank, but only 16 observations, lets take 
	the officer with the lowest appereances --> the one that change */

	bys officer_id: egen tempcount = count(officer_id)
	bys district date_treat (officer_id rank_designation): egen tempmaxcount = max(tempcount)	
	drop if temp2 == 2 & tempcount != tempmaxcount
	drop temp*
	bys district date_treat (officer_id rank_designation): gen temp = _n
	bys district date_treat: gen temp2 = _N // number of district officers per date

	* if there are still duplicates, its either a repetition or empty, so just lets drop them
	drop if temp2 == 2 & temp == 1
	drop if temp2 == 3 & temp < 3
	drop if temp2 == 4 & temp < 4
	drop temp*
	
	*complete panel dataset
	
	xtset district_code date_treat
	drop if date_treat == .
	tsfill, full
	
	*fix ids
	
	bys district_code (date_treat): replace officer_id = officer_id[_n-1] if officer_id[_n-1] !=. & officer_id == .
	decode district_code, gen(district_string)
	drop district_code date from
	drop if officer_id == .
	
	*create change of dc variable
	
	bys district (date_treat): gen change_dc = (officer_id != officer_id[_n-1])
	bys district (date_treat): gen temp = _n
	replace change_dc = 0 if temp == 1
	drop temp

	tempfile temp
	save `temp'
	restore

	*merge with main dataset
	
	joinby date_treat district_string using `temp'

	*keep if _merge == 3
	bys district officer_id (date_treat): gen duration = _N
	
	*label values
	label define meet 0 "Before" 1 "Meeting"
	label values meeting meet
	label define aftermeet 0 "Before" 1 "After meeting"
	label values aftermeeting aftermeet
	
	rename flagged flaggedbottom
	gen flaggedtop = flaggedbottom // trick to have the labels in the output table, but we only use the respective `d'
	
	label define fbottom 0 "Before" 1 "Bottom"
	label define ftop 0 "Before" 1 "Top"		
	label values flaggedbottom fbottom
	label values flaggedtop ftop

	*define panel structure
	egen group = group(stack district)
	egen time  = group(stack time_id)
	xtset group time
	
	*estimate regression for change in officers	
	set seed 12345			
	
	* full
	eststo full`d': xtreg change_dc i.flagged`d'##i.meeting i.flagged`d'##i.aftermeeting i.time, fe /// 
		vce(bootstrap)
			
		qui estadd ysumm
		qui sum change_dc if e(sample)
		qui estadd scalar yobs `r(N)'
		qui sum change_dc if e(sample) & relativetime == 0 
		qui estadd scalar ymeanbef `r(mean)'

	* rd
	eststo rd`d': xtreg change_dc i.flagged`d'##i.meeting i.flagged`d'##i.aftermeeting i.time ///
		if rd_`d' == 1 , fe /// 
		vce(bootstrap)
		
		qui estadd ysumm
		qui sum change_dc if e(sample)
		qui estadd scalar yobs `r(N)'
		qui sum change_dc if e(sample) & relativetime == 0 
		qui estadd scalar ymeanbef `r(mean)'

}

estout fullbottom rdbottom fulltop rdtop /// models
   using "$analysis/tables/causal/3_5_ranking_change.tex", /// file
   style(tex) /// style of exporting
   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
   mlabels(none) /// no label for models
   collabels(none) /// no label for columns
   keep(1.flaggedbottom#1.meeting 1.flaggedbottom#1.aftermeeting 1.flaggedtop#1.meeting 1.flaggedtop#1.aftermeeting) /// coefficients to keep
   order(1.flaggedbottom#1.meeting 1.flaggedbottom#1.aftermeeting 1.flaggedtop#1.meeting 1.flaggedtop#1.aftermeeting) /// order coefficients
   label interact("$\times$") /// interations
   prefoot(\midrule ) ///
   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
	labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
   replace // replace if it exist

*Table B10 

local datasets bottom top // Table B10
foreach d of local datasets{

	use "$analysis/processed/stacked/cc_stackdata_`d'_district.dta", clear

	/*drop duplicates as district outcomes are constant across markaz and schools, so we only keep 
	one observation per district/date/stack*/
	
	*duplicates drop district stack date_treat rd_`d' flagged avg_totalreportedteachers_dis ///
		*time_id relativetime, force
		
	collapse (mean) avg_totalreportedteachers_dis (max) flagged, by (district stack date_treat rd_`d'  ///
		time_id relativetime)

	*keep desired variables
	
	keep district stack date_treat rd_`d' flagged avg_totalreportedteachers_dis time_id relativetime
	
	*create meeting variables
	
	gen meeting = (time_id == 5)
	gen aftermeeting = (time_id > 5)
	decode district, gen(district_string)

	*here I clean a supporting dataset with the older and current ranking of district comissioners
	
	preserve 
	
	import delimited "$data\officers_punjab_pak.csv", clear 
	
	*drop unnecesary variables
	
	drop _merge dc 
	drop country province officer_federal
	rename from from1
	rename to from2
	drop if district == ""
	drop if officer_id == .
	
	*duplicates drop	
	bys district officer_id from1 from2: egen x = count(officer_id)
	drop if x == 2 & officer_id == 52 // duplicate district officer with one ranking empty
	drop x	
	
	sort district officer_id from1 from2
	gen random = runiform()
	sort district officer_id from1 from2 random
	bys district officer_id from1 from2: gen ord = _n
	keep if ord == 1
	drop random ord
		
	*gen id for district to reshape
	
	bys district: gen temp_id = _n
	reshape long from, i(district temp_id) j(date)
	
	*fix dates
	
	gen year = substr(from, 1, 4)
	destring year, replace
	gen month = substr(from, 6, 7)
	destring month, replace
	gen date_treat = ym(year, month)
	format date_treat %tm
	
	*standardize district codes to merge
	
	encode district, gen(district_code)
	drop district month year temp_id
	
	/*for each district date I need to assign only one district officer - in most of the cases 
	there are at least two district officers per date */	
	bys district date_treat (officer_id rank_designation): gen temp = _n
	bys district date_treat: gen temp2 = _N // number of district officers per date
	
	* keep the one with the highest rank
	bys district date_treat (officer_id rank_designation): egen tempmaxrank = min(rank_designation) if temp2 > 1
	drop if temp2 > 1 & rank_designation != tempmaxrank
	drop temp*
	bys district date_treat (officer_id rank_designation): gen temp = _n
	bys district date_treat: gen temp2 = _N // number of district officers per date
	
	/*now, there are some still repeated with the same rank, but only 16 observations, lets take 
	the officer with the lowest appereances --> the one that change */

	bys officer_id: egen tempcount = count(officer_id)
	bys district date_treat (officer_id rank_designation): egen tempmaxcount = max(tempcount)	
	drop if temp2 == 2 & tempcount != tempmaxcount
	drop temp*
	bys district date_treat (officer_id rank_designation): gen temp = _n
	bys district date_treat: gen temp2 = _N // number of district officers per date

	* if there are still duplicates, its either a repetition or empty, so just lets drop them
	drop if temp2 == 2 & temp == 1
	drop if temp2 == 3 & temp < 3
	drop if temp2 == 4 & temp < 4
	drop temp*
	
	*complete panel dataset
	
	xtset district_code date_treat
	drop if date_treat == .
	tsfill, full
	
	*fix ids
	
	bys district_code (date_treat): replace officer_id = officer_id[_n-1] if officer_id[_n-1] !=. & officer_id == .
	decode district_code, gen(district_string)
	drop district_code date from
	drop if officer_id == .
	
	*create change of dc variable
	
	bys district (date_treat): gen change_dc = (officer_id != officer_id[_n-1])
	bys district (date_treat): gen temp = _n
	replace change_dc = 0 if temp == 1
	drop temp

	tempfile temp
	save `temp'
	restore

	*merge with main dataset
	
	joinby date_treat district_string using `temp'

	*keep if _merge == 3
	bys district officer_id (date_treat): gen duration = _N
	*estimate regression for change in officers	

	set seed 12345			
	*estimate regression for rank employment
	
	preserve

	replace rank_designation = 0 if rank_designation == .
	bys officer_id: egen max_rank = max(rank_designation)
		
	bys officer_id district stack: egen mean_duration = mean(duration)
	keep if time_id == 4
	bys officer_id district stack: egen num_flag = sum(flagged)
	
	*treatment by officer id: number of times flagged
	
	*note we only use the respective `d', but define both as a trick for labellin in the table
	bys officer_id: egen nflagbottom = mean(num_flag)
	bys officer_id: egen nflagtop = mean(num_flag)
	
	*leave data at the officer id
	
	collapse (max) max_rank (mean) nflagbottom nflagtop, by(officer_id)

	*label var

	label var nflagbottom "Months in bottom district"
	label var nflagtop "Months in top district"
		
	* full
	eststo full`d': reg max_rank nflag`d', /// 
		vce(bootstrap)
			
		qui estadd ysumm
		qui sum max_rank if e(sample)
		qui estadd scalar yobs `r(N)'
		qui estadd scalar ymeanbef `r(mean)'

	restore

}

estout fullbottom fulltop /// models
   using "$analysis/tables/causal/3_5_ranking_ranking.tex", /// file
   style(tex) /// style of exporting
   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
   mlabels(none) /// no label for models
   collabels(none) /// no label for columns
   keep(nflagbottom nflagtop) /// coefficients to keep
   order(nflagbottom nflagtop) /// order coefficients
   label /// interations
   prefoot(\midrule ) ///
   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
	labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
   replace // replace if it exist

*---------------------------------------------------------------------------------------------------
**# 17. regressions: gaming of public officers #

* Table B6 Panel A

global treatments tpres satt fac 
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	bys stack mar: egen med = median(students) if relativetime == -1
	bys stack mar: egen maxmed = max(med)
	gen temp = students if relativetime == -1
	bys stack emis: egen maxstud = max(temp)
	gen bigger = (maxstud > maxmed)
	
	*variables for each period
	
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
	
	*outcome
	
	bys stack mar emis (time_id): gen ladminvisit = advis[_n+1] // probability of visit by aeo
	
	*label values
	
	label define f 0 "Before" 1 "Flag"
	label values flag f
	label define p 0 "Before" 1 "Punish"
	label values punish p
	label define a 0 "Before" 1 "After flag"
	label values afterflag a
	label define t 0 "No T" 1 "T"
	label values flagged t
	
	*label
	
	label var flag "Flag"
	label var punish "Punish"
	label var afterflag "After flag"
	label var flagged "T"
	
	* full regression
	eststo full`yvar': reghdfe ladminvisit i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if bigger == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum ladminvisit if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum ladminvisit if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

	* rd regression
	eststo rd`yvar': reghdfe ladminvisit i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if rd1 == 1 & bigger == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum ladminvisit if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum ladminvisit if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

}

local tabs "outcomes" 
foreach t of local tabs{
	
	if("`t'" == "outcomes"){
		
		local models fulltpres rdtpres fullsatt rdsatt fullfac rdfac
	}
	else{
		
		local models fullmath rdmath fulleng rdeng fullurdu rdurdu
	}
	
	*table
	
	estout `models' /// models
		   using "$analysis/tables/causal/10_flagging_`t'_admin_big.tex", /// file
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   keep(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// coefficients to keep
		   order(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// order coefficients
		   label interact("$\times$") /// interations
		   prefoot(\midrule ) ///
		   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
		   replace // replace if it exist

}

* Table B6 Panel B

global treatments tpres satt fac
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	bys stack mar: egen med = median(`yvar') if relativetime == -1
	bys stack mar: egen maxmed = max(med)
	gen temp = `yvar' if relativetime == -1
	bys stack emis: egen maxperf = max(temp)
	gen worse = (maxperf < maxmed)
	
	*variables for each period
	
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
	
	*outcome
	
	bys stack mar emis (time_id): gen ladminvisit = advis[_n+1] // probability of visit by aeo
	
	*label values
	
	label define f 0 "Before" 1 "Flag"
	label values flag f
	label define p 0 "Before" 1 "Punish"
	label values punish p
	label define a 0 "Before" 1 "After flag"
	label values afterflag a
	label define t 0 "No T" 1 "T"
	label values flagged t
	
	*label
	
	label var flag "Flag"
	label var punish "Punish"
	label var afterflag "After flag"
	label var flagged "T"
	
	* full regression
	eststo full`yvar': reghdfe ladminvisit i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if worse == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum ladminvisit if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum ladminvisit if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

	* rd regression
	eststo rd`yvar': reghdfe ladminvisit i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if rd1 == 1 & worse == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum ladminvisit if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum ladminvisit if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

}

local tabs "outcomes" 
foreach t of local tabs{
	
	if("`t'" == "outcomes"){
		
		local models fulltpres rdtpres fullsatt rdsatt fullfac rdfac
	}
	else{
		
		local models fullmath rdmath fulleng rdeng fullurdu rdurdu
	}
	
	*table
	
	estout `models' /// models
		   using "$analysis/tables/causal/10_flagging_`t'_admin_worse.tex", /// file
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   keep(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// coefficients to keep
		   order(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// order coefficients
		   label interact("$\times$") /// interations
		   prefoot(\midrule ) ///
		   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
		   replace // replace if it exist

}

* Table B6 Panel C

global treatments tpres satt fac 
foreach yvar of global treatments{
	
	*main data set
	
	use "$analysis/processed/stacked/cc_stackdata5_`yvar'.dta", clear
	bys stack mar: egen med = median(tpres) if relativetime == -1
	bys stack mar: egen maxmed = max(med)
	gen temp = tpres if relativetime == -1
	bys stack emis: egen maxperf = max(temp)
	gen mostmiss = (maxperf < maxmed)

	*variables for each period
	
	gen flag = (relativetime == 0)
	gen punish = (relativetime == 1)
	gen afterflag = (relativetime >= 2)
	
	*outcome
	
	bys stack mar emis (time_id): gen ladminvisit = advis[_n+1] // probability of visit by aeo
	
	*label values
	
	label define f 0 "Before" 1 "Flag"
	label values flag f
	label define p 0 "Before" 1 "Punish"
	label values punish p
	label define a 0 "Before" 1 "After flag"
	label values afterflag a
	label define t 0 "No T" 1 "T"
	label values flagged t
	
	*label
	
	label var flag "Flag"
	label var punish "Punish"
	label var afterflag "After flag"
	label var flagged "T"
	
	* full regression
	eststo full`yvar': reghdfe ladminvisit i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if mostmiss == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum ladminvisit if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum ladminvisit if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

	* rd regression
	eststo rd`yvar': reghdfe ladminvisit i.flagged##i.flag i.flagged##i.punish i.flagged##i.afterflag ///
		if rd1 == 1 & mostmiss == 1 [aw = avg_totalreportedteachers_mar], ///
		absorb(mar#stack time_id#stack district#stack#c.time_id) vce(cluster mar)
	qui estadd ysumm
	qui sum ladminvisit if e(sample)
	qui estadd scalar yobs `r(N)'
	qui sum ladminvisit if e(sample) & relativetime == 0 
	qui estadd scalar ymeanbef `r(mean)'

}

local tabs "outcomes" 
foreach t of local tabs{
	
	if("`t'" == "outcomes"){
		
		local models fulltpres rdtpres fullsatt rdsatt fullfac rdfac
	}
	else{
		
		local models fullmath rdmath fulleng rdeng fullurdu rdurdu
	}
	
	*table
	
	estout `models' /// models
		   using "$analysis/tables/causal/10_flagging_`t'_admin_missing.tex", /// file
		   style(tex) /// style of exporting
		   starl(* 0.10 ** 0.05 *** 0.01) /// significance stars
		   cells(b(star fmt(a2)) se(par fmt(a2))) /// format of cells (coefficient and se)
		   mlabels(none) /// no label for models
		   collabels(none) /// no label for columns
		   keep(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// coefficients to keep
		   order(1.flagged#1.flag 1.flagged#1.punish 1.flagged#1.afterflag) /// order coefficients
		   label interact("$\times$") /// interations
		   prefoot(\midrule ) ///
		   stats(N ymeanbef r2, fmt(%11.2gc a2 a2) ///
			labels("N. of obs." "Mean Dep. Var. before" "\$R^2\$")) /// stats to keep
		   replace // replace if it exist

}

